@@ -26,14 +26,14 @@ class TreesappTester(unittest.TestCase):
     def tearDown(self) -> None:
         # Clean up output directories
         output_prefix = os.path.join(os.path.abspath("./"), "TreeSAPP_")
-        for test_name in ["assign", "train", "update", "evaluate", "create", "package", "purity", "MCC", "colour"]:
+        for test_name in ["assign", "train", "update", "evaluate", "create", "package", "purity", "colour"]:
             output_dir = output_prefix + test_name
             if os.path.isdir(output_dir):
                 rmtree(output_dir)
         return
 
     def test_abundance(self):
-        from treesapp.commands import abundance
+        from treesapp.abundance import abundance
         from treesapp.file_parsers import read_classification_table
         from .testing_utils import get_test_data
         classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
@@ -46,12 +46,12 @@ class TreesappTester(unittest.TestCase):
                                   "--pairing", "pe",
                                   "--num_procs", str(self.num_procs),
                                   "--delete"]
-        abund_dict = abundance(abundance_command_list)
+        abund_dict = abundance(abundance_command_list)["test_TarA"]
         post_lines = read_classification_table(get_test_data(classification_table))
         # Ensure no lines were removed
         self.assertEqual(len(pre_lines), len(post_lines))
         # Ensure the name of the sample is substituted for the sample ID
-        self.assertEqual({"test_TarA.1"}, set([line[0] for line in post_lines]))
+        self.assertEqual({"test_TarA"}, set([line[0] for line in post_lines]))
         self.assertEqual(148, round(sum(abund_dict.values())))
         # Replace the classification table
         copyfile(os.path.join(self.ts_assign_output, "tmp.tsv"), classification_table)
@@ -60,7 +60,7 @@ class TreesappTester(unittest.TestCase):
 
     def test_assign_prot(self):
         ref_pkgs = ["McrA", "M0702", "S0001"]
-        from treesapp.commands import assign
+        from treesapp.assign import assign
         from treesapp.file_parsers import read_classification_table
         assign_commands_list = ["--fastx_input", self.aa_test_fa,
                                 "--targets", ','.join(ref_pkgs),
@@ -79,7 +79,8 @@ class TreesappTester(unittest.TestCase):
 
     def test_assign_dna(self):
         ref_pkgs = ["M0701", "M0702"]
-        from treesapp.commands import assign
+        from treesapp import assign
+        from .testing_utils import get_test_data
         from treesapp.file_parsers import read_classification_table
         assign_commands_list = ["--fastx_input", self.nt_test_fa,
                                 "--targets", ','.join(ref_pkgs),
@@ -88,7 +89,12 @@ class TreesappTester(unittest.TestCase):
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "strict",
                                 "--trim_align", "--overwrite", "--delete"]
-        assign(assign_commands_list)
+        assign.assign(assign_commands_list)
+
+        assign_commands_list += ["--rel_abund",
+                                 "--reads", get_test_data("SRR3669912_1.fastq"),
+                                 "--reverse", get_test_data("SRR3669912_2.fastq")]
+        assign.assign(assign_commands_list)
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
         self.assertEqual(6, len(lines))
         classified_seqs = [line[1].split()[0] for line in lines]
@@ -132,6 +138,7 @@ class TreesappTester(unittest.TestCase):
                                 "--min_taxonomic_rank", 'p',
                                 "--output", "./TreeSAPP_create",
                                 "--num_procs", str(self.num_procs),
+                                "--max_examples", str(100),
                                 "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
@@ -147,6 +154,7 @@ class TreesappTester(unittest.TestCase):
                                 "--output", "./TreeSAPP_create",
                                 "--refpkg_name", "PuhA",
                                 "--similarity", "0.90",
+                                "--bootstraps", str(4),
                                 "--profile", get_test_data("PuhA_search.hmm"),
                                 "--molecule", "prot",
                                 "--screen", "Bacteria,Archaea",
@@ -195,9 +203,12 @@ class TreesappTester(unittest.TestCase):
                                  "-m", "prot",
                                  "--taxonomic_ranks", "class", "order",
                                  "-n", str(self.num_procs),
-                                 "--trim_align", "--overwrite", "--delete"]
+                                 "--trim_align"]
         evaluate(evaluate_command_list)
-        self.assertEqual(True, True)
+
+        # Test rerunning of already processed data
+        evaluate(evaluate_command_list)
+        self.assertTrue(True)
         return
 
     def test_info(self):
@@ -268,22 +279,6 @@ class TreesappTester(unittest.TestCase):
             package(["rename", "-h"])
         return
 
-    def test_mcc_calculator(self):
-        from treesapp import mcc_calculator
-        from .testing_utils import get_test_data
-        cmd = ["--fastx_input", get_test_data("EggNOG_McrA.faa"),
-               "--annot_map", get_test_data("EggNOG_McrA_annot_map.tsv"),
-               "--output", "./TreeSAPP_MCC",
-               "--refpkg_dir", self.refpkg_dir,
-               "--targets", "McrA",
-               "--molecule", "prot",
-               "--tool", "treesapp",
-               "--num_procs", str(self.num_procs),
-               "--delete", "--svm", "--overwrite"]
-        mcc_calculator.mcc_calculator(cmd)
-        self.assertEqual(True, True)
-        return
-
     def test_train(self):
         import csv
         from treesapp.commands import train
