@@ -1,5 +1,6 @@
 import os
 import pytest
+import shutil
 import unittest
 
 from Bio import Entrez
@@ -7,7 +8,7 @@ from Bio import Entrez
 from . import testing_utils as utils
 
 
-class MyTestCase(unittest.TestCase):
+class EntrezUtilitiesTester(unittest.TestCase):
     def setUp(self) -> None:
         from treesapp.fasta import FASTA
         from treesapp.classy import Creator
@@ -19,8 +20,12 @@ class MyTestCase(unittest.TestCase):
         self.test_fa.keep_only(self.test_fa.get_seq_names()[0:18])
 
         self.create_inst = Creator()
+        self.create_inst.output_dir = "./tests/entrez_utils_test/"
+        self.create_inst.final_output_dir = self.create_inst.output_dir + "final_outputs" + os.sep
+        self.create_inst.var_output_dir = self.create_inst.output_dir + "intermediates" + os.sep
+        self.create_inst.check_previous_output()
         self.create_inst.current_stage = self.create_inst.stages[1]
-        self.create_inst.acc_to_lin = "./test_create_acc_to_lin.tsv"
+        self.create_inst.acc_to_lin = os.path.join(self.create_inst.output_dir, "test_create_acc_to_lin.tsv")
 
         self.accession2taxid = utils.get_test_data("create_test.accession2taxid")
         self.test_entrez_records = []
@@ -30,8 +35,17 @@ class MyTestCase(unittest.TestCase):
         return
 
     def tearDown(self) -> None:
-        if os.path.isfile(self.create_inst.acc_to_lin):
-            os.remove(self.create_inst.acc_to_lin)
+        if os.path.isdir(self.create_inst.output_dir):
+            shutil.rmtree(self.create_inst.output_dir)
+        return
+
+    def test_validate_target_db(self):
+        from treesapp.entrez_utils import validate_target_db
+        self.assertEqual("Taxonomy", validate_target_db("tax"))
+        self.assertEqual("nucleotide", validate_target_db("dna"))
+        with pytest.raises(SystemExit):
+            validate_target_db("nuc")
+        return
 
     def test_fetch_entrez_lineages(self):
         entrez_record_dict = self.create_inst.fetch_entrez_lineages(self.test_fa, 'prot')
@@ -107,6 +121,8 @@ class MyTestCase(unittest.TestCase):
         h2.first_split = h2.original
         h3 = Header('P060081_orf_3')
         h3.first_split = h3.original
+        h4 = Header('AB-746_I02_AB-902_NODE_1_length_133761_cov_569.254_ID_1_26 # 25226 # 25834 # 1 # ID=10766_26;partial=00;start_type=ATG;rbs_motif=AGGAG(G)/GGAGG;rbs_spacer=13-15bp;gc_cont=0.335')
+        h4.first_split = h4.original.split()[0]
 
         with pytest.raises(SystemExit):
             map_orf_lineages(seq_lineage_tbl=utils.get_test_data("SwissProt_PuhA_seqs2lineage.txt"), header_registry={})
@@ -119,12 +135,49 @@ class MyTestCase(unittest.TestCase):
         self.assertEqual("d__Bacteria; p__Proteobacteria; c__Alphaproteobacteria; o__Rhodobacterales;"
                          " f__Rhodobacteraceae; g__Rhodobacter; s__Rhodobacter sphaeroides",
                          orf_lin_map[h1.first_split])
+
+        # Test a complicated header
+        orf_lin_map, found = map_orf_lineages(seq_lineage_tbl=utils.get_test_data("test_seqs2lineage.txt"),
+                                              header_registry={'1': h4})
+        self.assertEqual(1, len(found))
+        return
+
+    def test_ref_seq_lineage_scanner(self):
+        from treesapp import entrez_utils
+        er_one = entrez_utils.EntrezRecord(".", ".")
+        er_one.lineage = 'r__Root; d__Bacteria; p__Tenericutes; c__Mollicutes; o__Mycoplasmatales;' \
+                         ' f__Mycoplasmataceae; g__Mycoplasmopsis; s__Mycoplasma cynos; t__Mycoplasma cynos C142'
+        matches = entrez_utils.ref_seq_lineage_scanner(ref_seq_dict={'1': er_one}, taxon_name="g__Mycoplasmopsis")
+        self.assertEqual([er_one], matches)
         return
 
     def test_repair_lineages(self):
-        # TODO: provide EntrezRecord instances in ref_seq_dict
-        from treesapp.entrez_utils import repair_lineages
-        repair_lineages(ref_seq_dict={}, t_hierarchy=self.create_inst.ref_pkg.taxa_trie)
+        from treesapp import entrez_utils
+        from treesapp.refpkg import ReferencePackage
+        # Load reference package to compare with
+        mcra_refpkg = ReferencePackage()
+        mcra_refpkg.f__json = utils.get_test_data(os.path.join("refpkgs", "McrA_build.pkl"))
+        mcra_refpkg.slurp()
+        # Set up mock test data
+        l1 = "Archaea; Euryarchaeota; environmental samples"
+        l2 = "Archaea; Euryarchaeota; Stenosarchaea group; Methanomicrobia; Methanosarcinales; " \
+             "Candidatus Methanoperedenaceae; Candidatus Methanoperedens; environmental samples"
+        l3 = "r__Root; d__Archaea; p__Candidatus Thermoplasmatota; c__Thermoplasmata_1; " \
+             "o__Methanomassiliicoccales_1; f__Candidatus Methanomethylophilaceae; " \
+             "g__Candidatus Methanomethylophilus; s__Candidatus Methanomethylophilus sp. 1R26"
+        mock_one_er = entrez_utils.EntrezRecord("AAW80308", "AAW80308.1")
+        mock_two_er = entrez_utils.EntrezRecord("QGT32507", "QGT32507.1")
+        mock_three_er = entrez_utils.EntrezRecord("seq_3", "seq_3.1")
+        mock_one_er.lineage = l1
+        mock_two_er.lineage = l2
+        mock_three_er.lineage = l3 + "; t__Methanomethylophilus sp. 1R26 strain mock"
+
+        # Test
+        entrez_utils.repair_lineages(ref_seq_dict={'1': mock_one_er, '2': mock_two_er, '3': mock_three_er},
+                                     t_hierarchy=mcra_refpkg.taxa_trie)
+        self.assertEqual("r__Root; d__Archaea; p__Euryarchaeota", mock_one_er.lineage)
+        self.assertEqual("r__Root; d__Archaea; p__Euryarchaeota", mock_two_er.lineage)
+        self.assertEqual(l3, mock_three_er.lineage)
         return
 
     def test_repair_conflict_lineages(self):
@@ -156,6 +209,11 @@ class MyTestCase(unittest.TestCase):
         taxonomy = TaxonomicHierarchy()
         taxonomy.feed_leaf_nodes(leaf_nodes)
 
+        # Retry with domain included
+        for leaf in leaf_nodes:
+            leaf.lineage = "d__Bacteria; " + leaf.lineage
+        taxonomy.feed_leaf_nodes(leaf_nodes)
+
         # Orders: 'Veillonellales' -> 'Selenomonadales', 'Selenomonadales' -> 'Clostridiales'
         # Families: 'Selenomonadaceae' -> 'Veillonellaceae'
         # Change the coverage of the different lineages to control the representative
@@ -166,10 +224,15 @@ class MyTestCase(unittest.TestCase):
                               (taxonomy.get_taxon("f__Veillonellaceae"), taxonomy.get_taxon("f__Selenomonadaceae"))}
 
         repair_conflict_lineages(taxonomy, er_dict)
-        self.assertEqual("p__Firmicutes; c__Clostridia; o__Clostridiales; f__Veillonellaceae; g__Megasphaera; s__Megasphaera elsdenii",
+        self.assertEqual("r__Root; d__Bacteria; p__Firmicutes; c__Clostridia; o__Clostridiales;"
+                         " f__Veillonellaceae; g__Megasphaera; s__Megasphaera elsdenii",
                          "; ".join([t.prefix_taxon() for t in taxonomy.get_taxon("s__Megasphaera elsdenii").lineage()]))
+        self.assertEqual("r__Root; d__Bacteria; p__Firmicutes; c__Clostridia; o__Clostridiales;"
+                         " f__Veillonellaceae; g__Mitsuokella; s__Mitsuokella multacida",
+                         "; ".join([t.prefix_taxon() for t in taxonomy.get_taxon("s__Mitsuokella multacida").lineage()]))
         self.assertFalse("o__Veillonellales" in taxonomy.hierarchy)
         self.assertFalse("o__Selenomonadales" in taxonomy.hierarchy)
+        self.assertFalse("c__Negativicutes" in taxonomy.hierarchy)
         return
 
     def test_verify_rank_occupancy(self):
