@@ -32,6 +32,7 @@ class ModuleFunction:
         """
         self.order = order
         self.name = name
+        self.dir_path = ""
         self.function = func
         self.run = True
 
@@ -326,6 +327,7 @@ class TreeSAPP:
         self.seq_lineage_map = dict()  # Dictionary holding the accession-lineage mapping information
         self.acc_to_lin = ""  # Path to an accession-lineage mapping file
         self.ref_pkg = ReferencePackage()
+        self.classification_tbl_name = "marker_contig_map.tsv"
 
         # Values derived from the command-line arguments
         self.input_sequences = ""
@@ -355,6 +357,12 @@ class TreeSAPP:
                                     "Input molecule type = " + self.molecule_type])
         return info_string
 
+    def set_output_dirs(self) -> None:
+        self.output_dir = validate_new_dir(self.output_dir)
+        self.final_output_dir = self.output_dir + "final_outputs" + os.sep
+        self.var_output_dir = self.output_dir + "intermediates" + os.sep
+        return
+
     def furnish_with_arguments(self, args) -> None:
         """
         Carries over the basic TreeSAPP arguments to the respective TreeSAPP-subclass.
@@ -364,9 +372,8 @@ class TreeSAPP:
         :return: None
         """
         if self.command != "info":
-            self.output_dir = validate_new_dir(args.output)
-            self.final_output_dir = self.output_dir + "final_outputs" + os.sep
-            self.var_output_dir = self.output_dir + "intermediates" + os.sep
+            self.output_dir = args.output
+            self.set_output_dirs()
             if set(vars(args)).issuperset({"molecule", "input"}):
                 self.input_sequences = args.input
                 self.molecule_type = args.molecule
@@ -374,7 +381,6 @@ class TreeSAPP:
                 if suffix1 == ".gz":
                     file_name, suffix2 = os.path.splitext(file_name)
                 self.sample_prefix = file_name
-                self.formatted_input = self.var_output_dir + self.sample_prefix + "_formatted.fasta"
 
         if self.command != "colour" and "pkg_path" in vars(args):
             if len(args.pkg_path) > 1:
@@ -421,6 +427,9 @@ class TreeSAPP:
         for output_dir in main_output_dirs:
             if not os.path.isdir(output_dir):
                 os.mkdir(output_dir)
+
+        for stage_order, stage in self.stages.items():  # type: (int, ModuleFunction)
+            stage.dir_path = self.var_output_dir + stage.name + os.sep
         return
 
     def log_progress(self):
@@ -469,23 +478,6 @@ class TreeSAPP:
         logging.error("No stages are set to run!\n")
         sys.exit(3)
 
-    # def read_progress_log(self):
-    #     """
-    #     Read the object's 'stage_file' and determine the completed stage
-    #
-    #     :return: An integer corresponding to the last completed stage's rank in 'stage_order'
-    #     """
-    #     completed_int = self.first_stage()
-    #     try:
-    #         progress_handler = open(self.stage_file)
-    #     except IOError:
-    #         logging.debug("Unable to open stage file '" + self.stage_file +
-    #                       "' for reading. Defaulting to stage " + str(completed_int) + ".\n")
-    #         return completed_int
-    #     # TODO: Finish this to enable continuing part-way through an analysis
-    #     progress_handler.close()
-    #     return completed_int
-
     def find_stage_dirs(self) -> int:
         """
         Selects the earliest checkpoint that the workflow can be started from.
@@ -500,7 +492,7 @@ class TreeSAPP:
         return 0
 
     def set_stage_dir(self) -> None:
-        self.stage_output_dir = os.path.join(self.var_output_dir, self.current_stage.name) + os.sep
+        self.stage_output_dir = self.current_stage.dir_path
         if not os.path.isdir(self.stage_output_dir):
             os.mkdir(self.stage_output_dir)
         return
@@ -512,9 +504,13 @@ class TreeSAPP:
         :return: None
         """
         # Find the next stage
-        self.current_stage.run = False
-        while not self.current_stage.run and self.current_stage.order < max(self.stages.keys()):
-            self.current_stage = self.stages[self.current_stage.order+1]
+        curr_order = self.current_stage.order
+        while curr_order < max(self.stages.keys()):
+            next_stage = self.stages[curr_order+1]  # type: ModuleFunction
+            if next_stage.run:
+                self.current_stage = next_stage
+                break
+            curr_order += 1
 
         # Update the output directory for this stage
         self.set_stage_dir()
@@ -554,8 +550,9 @@ class TreeSAPP:
         """
         # TODO: Summarise the steps to be taken and write to log
         for i, module in sorted(self.stages.items()):  # type: (int, ModuleFunction)
-            if os.path.isdir(os.path.join(self.var_output_dir, module.name)):
-                module.run = False
+            if os.path.isdir(module.dir_path):
+                if len(os.listdir(module.dir_path)) > 0:
+                    module.run = False
 
         if args.overwrite:
             last_valid_stage = self.first_stage()
@@ -564,7 +561,7 @@ class TreeSAPP:
 
         self.current_stage = self.stages[last_valid_stage]
         self.set_stage_dir()
-        if args.stage == "continue":
+        if "stage" not in vars(args) or args.stage == "continue":
             logging.debug("Continuing with stage '{}'\n".format(self.current_stage.name))
             self.edit_stages(last_valid_stage)
             return
@@ -586,7 +583,7 @@ class TreeSAPP:
 
         return
 
-    def find_executables(self, args):
+    def find_executables(self, args) -> dict:
         """
         Finds the executables in a user's path to alleviate the requirement of a sub_binaries directory
 
@@ -600,10 +597,8 @@ class TreeSAPP:
         if self.command == "abundance":
             dependencies += ["bwa"]
 
-        if self.command in ["create", "update", "train", "evaluate", "info"]:
-            dependencies += ["mmseqs", "mafft"]
-            if hasattr(args, "fast") and args.fast:
-                dependencies.append("FastTree")
+        if self.command in ["create", "update", "train", "evaluate", "info", "phylotu"]:
+            dependencies += ["mmseqs", "mafft", "FastTree"]
 
             if hasattr(args, "od_seq") and args.od_seq:
                 dependencies.append("OD-seq")
@@ -894,14 +889,35 @@ class Creator(TreeSAPP):
 
 class Purity(TreeSAPP):
     def __init__(self):
+        """Class instance for validating a reference package's purity with a reference sequence database"""
         super(Purity, self).__init__("purity")
-        self.assign_dir = ""
         self.classifications = ""
-        self.summarize_dir = ""
         self.metadata_file = ""
         self.assignments = None
-        self.stages = {0: ModuleFunction("assign", 0),
-                       1: ModuleFunction("summarize", 1)}
+        self.assign_jplace_file = ""
+        self.stages = {0: ModuleFunction("clean", 0),
+                       1: ModuleFunction("assign", 1),
+                       2: ModuleFunction("summarize", 2)}
+
+    def check_purity_arguments(self, args):
+        self.ref_pkg.f__json = args.pkg_path
+        self.ref_pkg.slurp()
+        self.refpkg_dir = os.path.dirname(self.ref_pkg.f__json)
+
+        self.formatted_input = self.stage_lookup("clean").dir_path + self.sample_prefix + "_formatted.fasta"
+
+        ##
+        # Define locations of files TreeSAPP outputs
+        ##
+        self.classifications = self.stage_lookup("assign").dir_path + "final_outputs" + os.sep + "marker_contig_map.tsv"
+        self.assign_jplace_file = os.path.join(self.stage_lookup("assign").dir_path, "iTOL_output",
+                                               self.ref_pkg.prefix, self.ref_pkg.prefix + "_complete_profile.jplace")
+        self.metadata_file = args.extra_info
+
+        if not os.path.isdir(self.var_output_dir):
+            os.makedirs(self.var_output_dir)
+
+        return
 
     def decide_stage(self, args):
         """
@@ -910,7 +926,6 @@ class Purity(TreeSAPP):
         This function ensures all the required inputs are present for beginning at the desired first stage,
         otherwise, the pipeline begins at the first possible stage to continue and ends once the desired stage is done.
 
-        If a
         :return: None
         """
         self.validate_continue(args)
@@ -1058,17 +1073,17 @@ class TaxonTest:
                                 "\t\n".join(off_targets[marker]) + "\n")
         return
     
-    def clade_exclusion_outputs(self, rank: str, refpkg: ReferencePackage, output_dir: str, tool: str) -> None:
+    def clade_exclusion_outputs(self, refpkg: ReferencePackage, output_dir: str, tool: str) -> None:
         """
         Creates a TaxonTest instance that stores file paths and settings relevant to a clade exclusion analysis
 
-        :param rank: The taxonomic rank (to which `lineage` belongs to) that is being excluded
         :param refpkg: The ReferencePackage object that is to be used for clade exclusion
         :param output_dir: Root directory for the various outputs for this TaxonTest
         :param tool: Name of the tool used for classifying query sequences: 'graft', 'diamond' or 'treesapp'
         :return: TaxonTest instance
         """
         taxon_path = re.sub(r"([ /])", '_', self.taxon)
+        taxon_path = re.sub(r"([()'\[\]])", '', taxon_path)
 
         self.intermediates_dir = os.path.join(output_dir, refpkg.prefix, taxon_path) + os.sep
         if not os.path.isdir(self.intermediates_dir):
@@ -1118,9 +1133,21 @@ class Evaluator(TreeSAPP):
         This function ensures all the required inputs are present for beginning at the desired first stage,
         otherwise, the pipeline begins at the first possible stage to continue and ends once the desired stage is done.
 
-        If a
         :return: None
         """
+        self.acc_to_lin = self.stage_lookup("lineages").dir_path + "accession_id_lineage_map.tsv"
+        if args.acc_to_lin:
+            if os.path.isfile(args.acc_to_lin):
+                self.acc_to_lin = args.acc_to_lin
+                self.change_stage_status("lineages", False)
+            else:
+                logging.error(
+                    "Unable to find accession-lineage mapping file '{}'\n".format(args.acc_to_lin))
+                sys.exit(3)
+        elif os.path.isfile(self.acc_to_lin):
+            logging.info("An accession-lineage mapping file from a previous run ('{}') was found"
+                         " and will attempt to be used.\n".format(self.acc_to_lin))
+            self.change_stage_status("lineages", False)
         self.validate_continue(args)
         return
 
@@ -1152,8 +1179,7 @@ class Evaluator(TreeSAPP):
         taxa_test_inst = TaxonTest(lineage)
         self.taxa_tests[rank].append(taxa_test_inst)
 
-        taxa_test_inst.clade_exclusion_outputs(output_dir=self.var_output_dir, refpkg=self.ref_pkg,
-                                               rank=rank, tool=tool)
+        taxa_test_inst.clade_exclusion_outputs(output_dir=self.var_output_dir, refpkg=self.ref_pkg, tool=tool)
         
         return taxa_test_inst
 
@@ -1437,11 +1463,16 @@ class Layerer(TreeSAPP):
 class Abundance(TreeSAPP):
     def __init__(self):
         super(Abundance, self).__init__("abundance")
-        self.stages = {}
         self.target_refpkgs = list()
         self.classified_nuc_seqs = ""
         self.classifications = ""
-        self.fq_suffix_re = re.compile(r"([._-])+(pe|fq|fastq|fwd|R1)$")
+        self.aln_file = ""
+        self.append_abundance = False
+        self.fq_suffix_re = re.compile(r"([._-])+(pe|fq|fastq|fwd|R1|1)$")
+        self.stages = {0: ModuleFunction("align_map", 0),
+                       1: ModuleFunction("sam_sum", 1),
+                       2: ModuleFunction("summarise", 2)}
+        return
 
     def check_arguments(self, args):
         ##
@@ -1451,13 +1482,53 @@ class Abundance(TreeSAPP):
             logging.error("Unable to find classified ORF nucleotide sequences in '{}'.\n".format(self.final_output_dir))
             sys.exit(5)
         self.classified_nuc_seqs = glob(self.final_output_dir + "*_classified.fna")[0]
-        if not os.path.isfile(self.classified_nuc_seqs):
-            logging.error("Unable to find classified sequences FASTA file in %s.\n" % self.final_output_dir)
-        self.classifications = self.output_dir + "final_outputs" + os.sep + "marker_contig_map.tsv"
+
+        self.classifications = self.final_output_dir + self.classification_tbl_name
 
         if not os.path.isdir(self.var_output_dir):
             os.makedirs(self.var_output_dir)
 
+        self.aln_file = self.stage_lookup("align_map").dir_path + \
+                        '.'.join(os.path.basename(self.classified_nuc_seqs).split('.')[0:-1]) + ".sam"
+
+        if len(args.reverse) > 0:
+            if len(args.reads) != len(args.reverse):
+                logging.error("Number of fastq files differs between reads () and reverse () arguments!.\n"
+                              "".format(len(args.reads), len(args.reverse)))
+                sys.exit(3)
+
+        if args.report == "update":
+            self.append_abundance = False
+
+        return
+
+    def decide_stage(self, args):
+        """
+        Bases the stage(s) to run on args.stage which is broadly set to either 'continue' or any other valid stage
+
+        This function ensures all the required inputs are present for beginning at the desired first stage,
+        otherwise, the pipeline begins at the first possible stage to continue and ends once the desired stage is done.
+
+        :return: None
+        """
+        self.validate_continue(args)
+        return
+
+    def delete_intermediates(self, clean_up: bool) -> None:
+        if not clean_up:
+            return
+
+        files_to_be_deleted = [self.aln_file]
+        files_to_be_deleted += glob(self.stage_lookup("align_map").dir_path + "*.stderr")
+
+        for file_path in files_to_be_deleted:
+            if os.path.exists(file_path):
+                os.remove(file_path)
+        return
+
+    def strip_file_to_sample_name(self, file_path) -> None:
+        file_prefix = '.'.join(os.path.basename(file_path).split('.')[:-1])
+        self.sample_prefix = self.fq_suffix_re.sub('', file_prefix)
         return
 
 
