@@ -4,12 +4,10 @@ import sys
 import re
 import os
 import shutil
-from random import randint
 
+import tqdm
 from joblib import dump as jdump
 from joblib import load as jload
-from collections import namedtuple
-from samsum import commands as samsum_cmd
 
 from treesapp import entrez_utils
 from treesapp import file_parsers
@@ -131,7 +129,7 @@ Use '-h' to get subcommand-specific help, e.g. 'treesapp package view -h'
     if not os.path.isdir(args.output):
         os.mkdir(args.output)
 
-    classy.prep_logging(log_file=os.path.join(args.output, 'TreeSAPP_package_log.txt'), stream=sys.stdout)
+    classy.prep_logging(log_file=os.path.join(args.output, 'TreeSAPP_package_log.txt'))
 
     for refpkg_pkl in args.pkg_path:
         refpkg.f__json = refpkg_pkl
@@ -281,8 +279,8 @@ def train(sys_args):
             assign_params.append("--trim_align")
 
         try:
-            assign(assign_params)
-            plain_pqueries = ts_phylo_seq.assignments_to_treesaps(file_parsers.read_classification_table(
+            ts_assign_mod.assign(assign_params)
+            plain_pqueries = ts_phylo_seq.assignments_to_pqueries(file_parsers.read_classification_table(
                 os.path.join(assign_prefix, "final_outputs", "marker_contig_map.tsv")))
         except (SystemExit, IOError):
             logging.info("failed.\n")
@@ -494,6 +492,7 @@ def create(sys_args):
     # Optionally cluster the input sequences using MMSeqs' linclust at the specified identity
     ##
     if ts_create.stage_status("cluster"):
+        # TODO: replace with fasta.dereplicate_by_clustering()
         pre_cluster = ref_seqs.n_seqs()
         ref_seqs.change_dict_keys("num")
         # Write a FASTA for clustering containing the formatted headers since
@@ -507,8 +506,7 @@ def create(sys_args):
                                       similarity=ts_create.ref_pkg.pid, num_threads=args.num_threads)
             ts_create.clusters_table = ts_create.clusters_prefix + "_cluster.tsv"
             cluster_alignments = ts_create.clusters_prefix + "_cluster_aln.tsv"
-        # Read the uc file if present
-        if ts_create.clusters_table and cluster_alignments:
+
             cluster_dict = file_parsers.create_mmseqs_clusters(ts_create.clusters_table, cluster_alignments)
 
             # Revert headers in cluster_dict from 'formatted' back to 'original'
@@ -517,7 +515,7 @@ def create(sys_args):
             ##
             # Calculate LCA of each cluster to represent the taxonomy of the representative sequence
             ##
-            ts_create_mod.cluster_lca(cluster_dict, fasta_records, ref_seqs.header_registry)
+            ts_create_mod.find_cluster_lca(cluster_dict, fasta_records, ref_seqs.header_registry)
         else:
             cluster_dict = None
 
@@ -537,7 +535,7 @@ def create(sys_args):
         if ts_create.clusters_table and not args.headless:
             # Allow user to select the representative sequence based on organism name, sequence length and similarity
             ts_create_mod.present_cluster_rep_options(cluster_dict, fasta_records, ref_seqs.header_registry,
-                                                      ref_seqs.amendments)
+                                                      important_seqs=ref_seqs.amendments)
         elif ts_create.clusters_table and args.headless:
             ts_create_mod.finalize_cluster_reps(cluster_dict, fasta_records, ref_seqs.header_registry)
         else:
@@ -586,8 +584,8 @@ def create(sys_args):
 
         if args.multiple_alignment is False:
             logging.info("Aligning the sequences using MAFFT... ")
-            ts_create_mod.run_mafft(ts_create.executables["mafft"],
-                                    ts_create.unaln_ref_fasta, ts_create.ref_pkg.f__msa, args.num_threads)
+            wrapper.run_mafft(ts_create.executables["mafft"], ts_create.unaln_ref_fasta,
+                              ts_create.ref_pkg.f__msa, args.num_threads)
             logging.info("done.\n")
         else:
             pass
@@ -656,6 +654,8 @@ def create(sys_args):
         ts_create.determine_model(ts_create.ref_pkg)
         best_tree = ts_create.ref_pkg.infer_phylogeny(ts_create.phylip_file, ts_create.executables, ts_create.phy_dir,
                                                       args.num_threads)
+    else:
+        best_tree = "{}{}.{}.nwk".format(ts_create.phy_dir, ts_create.ref_pkg.prefix, ts_create.ref_pkg.tree_tool)
 
     if ts_create.stage_status("evaluate"):
         # Evaluate the model parameters with RAxML-NG. Output is required by EPA-NG
@@ -669,7 +669,7 @@ def create(sys_args):
         # Perform non-parametric bootstrapping with RAxML-NG and calculate branch support values from bootstraps
         wrapper.support_tree_raxml(raxml_exe=ts_create.executables["raxml-ng"], model=ts_create.ref_pkg.sub_model,
                                    ref_tree=best_tree, ref_msa=ts_create.phylip_file,
-                                   tree_prefix=ts_create.phy_dir + ts_create.ref_pkg.prefix,
+                                   tree_prefix=os.path.join(ts_create.phy_dir, ts_create.ref_pkg.prefix),
                                    mre=True, n_bootstraps=args.bootstraps, num_threads=args.num_threads)
         ts_create.ref_pkg.recover_raxmlng_supports(ts_create.phy_dir)
 
@@ -680,7 +680,9 @@ def create(sys_args):
                    "-o", ts_create.training_dir,
                    "-m", ts_create.ref_pkg.molecule,
                    "-a", ts_create.acc_to_lin,
-                   "-n", str(args.num_threads)]
+                   "--num_procs", str(args.num_threads),
+                   "--max_examples", str(args.max_examples),
+                   "--svm_kernel", args.kernel]
     if args.trim_align:
         trainer_cmd.append("--trim_align")
 
@@ -851,7 +853,7 @@ def update(sys_args):
 
         # Calculate LCA of each cluster to represent the taxonomy of the representative sequence
         entrez_records = ts_update_mod.simulate_entrez_records(combined_fasta, classified_seq_lineage_map)
-        ts_create_mod.cluster_lca(cluster_dict, entrez_records, combined_fasta.header_registry)
+        ts_create_mod.find_cluster_lca(cluster_dict, entrez_records, combined_fasta.header_registry)
 
         # Ensure centroids are the original reference sequences and skip clusters with identical lineages
         ts_update_mod.prefilter_clusters(cluster_dict,
@@ -866,7 +868,7 @@ def update(sys_args):
         else:
             # Allow user to select the representative sequence based on organism name, sequence length and similarity
             ts_create_mod.present_cluster_rep_options(cluster_dict, entrez_records, combined_fasta.header_registry,
-                                                      combined_fasta.amendments, True)
+                                                      important_seqs=combined_fasta.amendments, each_lineage=True)
 
         # Remove sequences that were replaced by resolve from ts_updater.old_ref_fasta
         still_repping = []
@@ -1107,210 +1109,6 @@ def layer(sys_args):
     return
 
 
-def assign(sys_args):
-    # STAGE 1: Prompt the user and prepare files and lists for the pipeline
-    parser = treesapp_args.TreeSAPPArgumentParser(description='Classify sequences through evolutionary placement.')
-    treesapp_args.add_classify_arguments(parser)
-    args = parser.parse_args(sys_args)
-
-    ts_assign = ts_assign_mod.Assigner()
-    ts_assign.furnish_with_arguments(args)
-    ts_assign.check_previous_output(args.overwrite)
-
-    log_file_name = args.output + os.sep + "TreeSAPP_classify_log.txt"
-    classy.prep_logging(log_file_name, args.verbose)
-    logging.info("\n##\t\t\t\tAssigning sequences with TreeSAPP\t\t\t\t##\n\n")
-
-    treesapp_args.check_parser_arguments(args, sys_args)
-    ts_assign.check_classify_arguments(args)
-    ts_assign.decide_stage(args)
-
-    refpkg_dict = file_parsers.gather_ref_packages(ts_assign.refpkg_dir, ts_assign.target_refpkgs)
-    ts_assign_mod.prep_reference_packages_for_assign(refpkg_dict, ts_assign.var_output_dir)
-    ref_alignment_dimensions = ts_assign_mod.get_alignment_dims(refpkg_dict)
-
-    ##
-    # STAGE 2: Predict open reading frames (ORFs) if the input is an assembly, read, format and write the FASTA
-    ##
-    if ts_assign.stage_status("orf-call"):
-        ts_assign.predict_orfs(args.composition, args.num_threads)
-
-    query_seqs = fasta.FASTA(ts_assign.query_sequences)
-    # Read the query sequences provided and (by default) write a new FASTA file with formatted headers
-    if ts_assign.stage_status("clean"):
-        logging.info("Reading and formatting {}... ".format(ts_assign.query_sequences))
-        query_seqs.header_registry = fasta.format_fasta(fasta_input=ts_assign.query_sequences, molecule="prot",
-                                                        output_fasta=ts_assign.formatted_input)
-        logging.info("done.\n")
-    else:
-        ts_assign.formatted_input = ts_assign.query_sequences
-        query_seqs.load_fasta()
-    logging.info("\tTreeSAPP will analyze the " + str(len(query_seqs.header_registry)) + " sequences found in input.\n")
-
-    ##
-    # STAGE 3: Run hmmsearch on the query sequences to search for marker homologs
-    ##
-    if ts_assign.stage_status("search"):
-        hmm_domtbl_files = wrapper.hmmsearch_orfs(ts_assign.executables["hmmsearch"],
-                                                  refpkg_dict, ts_assign.formatted_input,
-                                                  ts_assign.var_output_dir, args.num_threads, args.max_e)
-        hmm_matches = file_parsers.parse_domain_tables(args, hmm_domtbl_files)
-        ts_assign_mod.load_homologs(hmm_matches, ts_assign.formatted_input, query_seqs)
-        pqueries = ts_assign_mod.load_pqueries(hmm_matches, query_seqs)
-        query_seqs.change_dict_keys("num")
-        extracted_seq_dict, numeric_contig_index = ts_assign_mod.bin_hmm_matches(hmm_matches, query_seqs.fasta_dict)
-        numeric_contig_index = ts_assign_mod.replace_contig_names(numeric_contig_index, query_seqs)
-        homolog_seq_files = ts_assign_mod.write_grouped_fastas(extracted_seq_dict, numeric_contig_index,
-                                                               refpkg_dict, ts_assign.var_output_dir)
-        # TODO: Replace this merge_fasta_dicts_by_index with FASTA - only necessary for writing the classified sequences
-        extracted_seq_dict = fasta.merge_fasta_dicts_by_index(extracted_seq_dict, numeric_contig_index)
-        ts_assign_mod.delete_files(args.delete, ts_assign.var_output_dir, 1)
-    ##
-    # STAGE 4: Run hmmalign, and optionally BMGE, to produce the MSAs for phylogenetic placement
-    ##
-    combined_msa_files = dict()
-    split_msa_files = dict()
-    if ts_assign.stage_status("align"):
-        ts_assign_mod.create_ref_phy_files(refpkg_dict, ts_assign.var_output_dir,
-                                           homolog_seq_files, ref_alignment_dimensions)
-        concatenated_msa_files = ts_assign_mod.multiple_alignments(ts_assign.executables, homolog_seq_files,
-                                                                   refpkg_dict, "hmmalign", args.num_threads)
-        file_type = utilities.find_msa_type(concatenated_msa_files)
-        alignment_length_dict = ts_assign_mod.get_sequence_counts(concatenated_msa_files, ref_alignment_dimensions,
-                                                                  args.verbose, file_type)
-
-        if args.trim_align:
-            tool = "BMGE"
-            trimmed_mfa_files = wrapper.filter_multiple_alignments(ts_assign.executables, concatenated_msa_files,
-                                                                   refpkg_dict, args.num_threads, tool)
-            qc_ma_dict = ts_assign_mod.check_for_removed_sequences(trimmed_mfa_files, concatenated_msa_files,
-                                                                   refpkg_dict, args.min_seq_length)
-            ts_assign_mod.evaluate_trimming_performance(qc_ma_dict, alignment_length_dict, concatenated_msa_files, tool)
-            combined_msa_files.update(qc_ma_dict)
-        else:
-            combined_msa_files.update(concatenated_msa_files)
-
-        # Subset the multiple alignment of reference sequences and queries to just contain query sequences
-        MSAs = namedtuple("MSAs", "ref query")
-        for denominator in combined_msa_files:
-            split_msa_files[denominator] = []
-            for combined_msa in combined_msa_files[denominator]:
-                split_msa = MSAs(os.path.dirname(combined_msa) + os.sep +
-                                 os.path.basename('.'.join(combined_msa.split('.')[:-1])) + "_references.mfa",
-                                 os.path.dirname(combined_msa) + os.sep +
-                                 os.path.basename('.'.join(combined_msa.split('.')[:-1])) + "_queries.mfa")
-                fasta.split_combined_ref_query_fasta(combined_msa, split_msa.query, split_msa.ref)
-                split_msa_files[denominator].append(split_msa)
-        combined_msa_files.clear()
-        ts_assign_mod.delete_files(args.delete, ts_assign.var_output_dir, 3)
-
-    ##
-    # STAGE 5: Run EPA-ng to compute the ML estimations
-    ##
-    if ts_assign.stage_status("place"):
-        wrapper.launch_evolutionary_placement_queries(ts_assign.executables, split_msa_files, refpkg_dict,
-                                                      ts_assign.var_output_dir, args.num_threads)
-        jplace_utils.sub_indices_for_seq_names_jplace(ts_assign.var_output_dir, numeric_contig_index, refpkg_dict)
-
-    if ts_assign.stage_status("classify"):
-        itol_out_dir = ts_assign.output_dir + 'iTOL_output' + os.sep
-        tree_saps, itol_data = ts_assign_mod.parse_raxml_output(ts_assign.var_output_dir, refpkg_dict, pqueries)
-        # Set PQuery.consensus_placement attributes
-        ts_assign_mod.select_query_placements(tree_saps, refpkg_dict, mode=args.p_sum)
-        ts_assign_mod.filter_placements(tree_saps, refpkg_dict, ts_assign.svc_filter, args.min_lwr)
-        ts_assign_mod.determine_confident_lineage(tree_saps, refpkg_dict)
-
-        ts_assign.write_classified_orfs(tree_saps, extracted_seq_dict)
-        abundance_dict = dict()
-        for refpkg_code in tree_saps:
-            for placed_seq in tree_saps[refpkg_code]:  # type: ts_phylo_seq.PQuery
-                abundance_dict[placed_seq.place_name] = 1.0
-
-        # Run the abundance subcommand on the classified sequences
-        if ts_assign.stage_status("abundance"):
-            abundance_args = ["--treesapp_output", ts_assign.output_dir,
-                              "--reads", args.reads,
-                              "--pairing", args.pairing,
-                              "--num_procs", str(args.num_threads),
-                              "--report", "nothing"]
-            if args.reverse:
-                abundance_args += ["--reverse", args.reverse]
-            abundance_dict = abundance(abundance_args)
-        ts_assign_mod.abundify_tree_saps(tree_saps, abundance_dict)
-
-        ts_assign_mod.write_classification_table(tree_saps, ts_assign.sample_prefix,
-                                                 output_file=ts_assign.final_output_dir + os.sep + "marker_contig_map.tsv")
-
-        ts_assign_mod.produce_itol_inputs(tree_saps, refpkg_dict, itol_data, itol_out_dir, ts_assign.refpkg_dir)
-        ts_assign_mod.delete_files(args.delete, ts_assign.var_output_dir, 4)
-
-    ts_assign_mod.delete_files(args.delete, ts_assign.var_output_dir, 5)
-
-    return
-
-
-def abundance(sys_args):
-    """
-    TreeSAPP subcommand that is used to add read-inferred abundance information (e.g. FPKM, TPM) to classified sequences
-    Command requires:
-
-1. Path to TreeSAPP output directory that contains classified sequences (FASTA-format) in the final_outputs/
-2. Path to read file(s) in FASTQ format
-3. Parameters indicating whether a) the reads are paired-end or single-end and b) the FASTQ is interleaved
-
-    With these arguments and the option `--report update` TreeSAPP would run BWA MEM and samsum to
-    calculate the desired abundance values (FPKM by default) and update the classification table with the
-    Sample (first column) of the classification table matching the prefix of the FASTQ.
-
-    Optionally, `treesapp abundance` can be called with `--report nothing` and a dictionary containing the abundance
-    values would be returned.
-
-    :param sys_args: treesapp abundance arguments with the treesapp subcommand removed
-    :return: A dictionary containing the abundance values indexed by the reference sequence (e.g. ORF, contig) names
-    """
-    parser = treesapp_args.TreeSAPPArgumentParser(description="Calculate classified sequence abundances from read coverage.")
-    treesapp_args.add_abundance_arguments(parser)
-    args = parser.parse_args(sys_args)
-
-    ts_abund = classy.Abundance()
-    ts_abund.furnish_with_arguments(args)
-
-    log_file_name = args.output + os.sep + "TreeSAPP_abundance_log.txt"
-    classy.prep_logging(log_file_name, args.verbose)
-    logging.info("\n##\t\t\tCalculating abundance of classified sequences\t\t\t##\n")
-
-    treesapp_args.check_parser_arguments(args, sys_args)
-    ts_abund.check_arguments(args)
-    # TODO: Implement check-pointing for abundance
-    # ts_abund.decide_stage(args)
-
-    sam_file = ts_assign_mod.align_reads_to_nucs(ts_abund.executables["bwa"], ts_abund.classified_nuc_seqs,
-                                                 ts_abund.var_output_dir, args.reads, args.pairing, args.reverse,
-                                                 args.num_threads)
-    ts_abund.sample_prefix = ts_abund.fq_suffix_re.sub('', '.'.join(os.path.basename(args.reads).split('.')[:-1]))
-
-    if os.path.isfile(sam_file):
-        ref_seq_abunds = samsum_cmd.ref_sequence_abundances(aln_file=sam_file, seq_file=ts_abund.classified_nuc_seqs,
-                                                            min_aln=10, p_cov=50, map_qual=1, multireads=False)
-        abundance_dict = {ref_seq.name: ref_seq.fpkm for ref_seq in ref_seq_abunds.values()}
-        ref_seq_abunds.clear()
-    else:
-        logging.warning("SAM file '%s' was not generated.\n" % sam_file)
-        return {}
-
-    ts_assign_mod.delete_files(args.delete, ts_abund.var_output_dir, 4)
-
-    # TODO: Index each PQuery's abundance by the dataset name, write a new row for each dataset's abundance
-    if args.report != "nothing" and os.path.isfile(ts_abund.classifications):
-        assignments = file_parsers.read_classification_table(ts_abund.classifications)
-        # Convert assignments to PQuery instances
-        pqueries = ts_phylo_seq.assignments_to_treesaps(assignments)
-        ts_assign_mod.abundify_tree_saps(pqueries, abundance_dict)
-        ts_assign_mod.write_classification_table(pqueries, ts_abund.sample_prefix, ts_abund.classifications)
-
-    return abundance_dict
-
-
 def purity(sys_args):
     """
 
@@ -1330,18 +1128,20 @@ def purity(sys_args):
     logging.info("\n##\t\t\tBeginning purity analysis\t\t\t##\n")
 
     treesapp_args.check_parser_arguments(args, sys_args)
-    treesapp_args.check_purity_arguments(ts_purity, args)
+    ts_purity.check_purity_arguments(args)
     ts_purity.decide_stage(args)
 
     # Load FASTA data
-    ref_seqs = fasta.FASTA(ts_purity.input_sequences)
-    ref_seqs.load_fasta()
-    ref_seqs.change_dict_keys()
-    ref_seqs.unalign()
-    fasta.write_new_fasta(ref_seqs.fasta_dict, ts_purity.formatted_input)
+    if ts_purity.stage_status("clean"):
+        ref_seqs = fasta.FASTA(ts_purity.input_sequences)
+        ref_seqs.load_fasta()
+        ref_seqs.change_dict_keys()
+        ref_seqs.unalign()
+        fasta.write_new_fasta(ref_seqs.fasta_dict, ts_purity.formatted_input)
+        ts_purity.increment_stage_dir()
 
     if ts_purity.stage_status("assign"):
-        assign_args = ["-i", ts_purity.formatted_input, "-o", ts_purity.assign_dir,
+        assign_args = ["-i", ts_purity.formatted_input, "-o", ts_purity.stage_output_dir,
                        "-m", ts_purity.molecule_type, "-n", str(args.num_threads),
                        "-t", ts_purity.ref_pkg.prefix, "--refpkg_dir", ts_purity.refpkg_dir,
                        "--overwrite", "--delete"]
@@ -1349,9 +1149,10 @@ def purity(sys_args):
             assign_args.append("--trim_align")
 
         try:
-            assign(assign_args)
+            ts_assign_mod.assign(assign_args)
         except:  # Just in case treesapp assign fails, just continue
             logging.error("TreeSAPP failed.\n")
+        ts_purity.increment_stage_dir()
 
     if ts_purity.stage_status("summarize"):
         metadat_dict = dict()
@@ -1370,9 +1171,7 @@ def purity(sys_args):
         if ts_purity.metadata_file:
             metadat_dict.update(ts_purity.load_metadata())
         # Identify the number of sequences that are descendents of each orthologous group
-        jplace_file = os.sep.join([ts_purity.assign_dir, "iTOL_output", ts_purity.ref_pkg.prefix,
-                                   ts_purity.ref_pkg.prefix + "_complete_profile.jplace"])
-        jplace_data = jplace_utils.jplace_parser(jplace_file)
+        jplace_data = jplace_utils.jplace_parser(ts_purity.assign_jplace_file)
         jplace_data.pqueries = jplace_utils.demultiplex_pqueries(jplace_data)
         node_map = entish.map_internal_nodes_leaves(jplace_data.tree)
         labelled_ref_tree = ts_purity.ref_pkg.taxonomically_label_tree()
@@ -1421,16 +1220,9 @@ def evaluate(sys_args):
 
     # Load FASTA data
     query_fasta = fasta.FASTA(args.input)
-    query_fasta.fasta_dict = fasta.format_read_fasta(query_fasta.file, args.molecule)
+    query_fasta.load_fasta(format_it=True, molecule=args.molecule)
     if args.length:
-        for seq_id in query_fasta.fasta_dict:
-            if len(query_fasta.fasta_dict[seq_id]) < args.length:
-                logging.warning(seq_id + " sequence is shorter than " + str(args.length) + "\n")
-            else:
-                max_stop = len(query_fasta.fasta_dict[seq_id]) - args.length
-                random_start = randint(0, max_stop)
-                query_fasta.fasta_dict[seq_id] = query_fasta.fasta_dict[seq_id][random_start:random_start + args.length]
-    query_fasta.header_registry = fasta.register_headers(fasta.get_headers(query_fasta.file))
+        query_fasta.trim_to_length(args.length)
 
     fasta_records = ts_evaluate.fetch_entrez_lineages(query_fasta, args.molecule, args.acc_to_taxid)
     entrez_utils.fill_ref_seq_lineages(fasta_records, ts_evaluate.seq_lineage_map)
@@ -1445,112 +1237,55 @@ def evaluate(sys_args):
     representative_seqs, ts_evaluate.taxa_filter = ts_clade_ex.pick_taxonomic_representatives(fasta_records,
                                                                                               ts_evaluate.taxa_filter)
     logging.info("done.\n")
+    logging.info("\t{} representative sequences may be used by TreeSAPP evaluate.\n".format(len(representative_seqs)))
 
-    logging.info("\t{} representative sequences will be used by TreeSAPP evaluate.\n".format(len(representative_seqs)))
-
-    deduplicated_fasta_dict = ts_clade_ex.select_rep_seqs(representative_seqs, fasta_records)
+    deduplicated_fasta_dict = ts_clade_ex.select_rep_seqs(representative_seqs,
+                                                          test_sequences=fasta_records,
+                                                          taxon_hierarchy=ts_evaluate.ref_pkg.taxa_trie)
     fasta.write_new_fasta(deduplicated_fasta_dict, ts_evaluate.test_rep_taxa_fasta)
     rep_accession_lineage_map = ts_clade_ex.map_seqs_to_lineages(ts_evaluate.seq_lineage_map, deduplicated_fasta_dict)
 
+    # Ensure that both lineage sets are rooted
+    ts_clade_ex.check_lineage_compatibility(ref_lineages, ts_evaluate.ref_pkg.taxa_trie)
+    ts_clade_ex.check_lineage_compatibility(rep_accession_lineage_map, ts_evaluate.ref_pkg.taxa_trie)
     # Checkpoint three: We have accessions linked to taxa, and sequences to analyze with TreeSAPP, but not classified
     if ts_evaluate.stage_status("classify"):
         # Run TreeSAPP against the provided tax_ids file and the unique taxa FASTA file
         for rank in args.taxon_rank:
-            for lineage in ts_clade_ex.get_testable_lineages_for_rank(ref_lineages, rep_accession_lineage_map,
-                                                                      rank, ts_evaluate.ref_pkg.taxa_trie):
+            candidate_lineages = ts_clade_ex.get_testable_lineages_for_rank(ref_lineages, rep_accession_lineage_map,
+                                                                            rank, ts_evaluate.ref_pkg.taxa_trie)
+            p_bar = tqdm.tqdm(ncols=100, desc="Evaluating {}".format(rank), total=len(candidate_lineages))
+            for lineage in candidate_lineages:
                 # Select representative sequences belonging to the taxon being tested
-                taxon_rep_seqs = ts_clade_ex.select_rep_seqs(representative_seqs, fasta_records, lineage)
+                taxon_rep_seqs = ts_clade_ex.select_rep_seqs(representative_seqs, fasta_records,
+                                                             taxon_hierarchy=ts_evaluate.ref_pkg.taxa_trie,
+                                                             target_lineage=lineage)
                 # Decide whether to continue analyzing taxon based on number of query sequences
                 if len(taxon_rep_seqs.keys()) == 0:
                     logging.debug("No query sequences for {}.\n".format(lineage))
+                    p_bar.update()
                     continue
 
                 # Continuing with classification
                 tt_obj = ts_evaluate.new_taxa_test(lineage, args.tool)
                 tt_obj.queries = taxon_rep_seqs.keys()
 
-                logging.info("Classifications for '{}' put in {}\n".format(tt_obj.taxon, tt_obj.intermediates_dir))
-
+                logging.debug("Classifications for '{}' put in {}\n".format(tt_obj.taxon, tt_obj.intermediates_dir))
                 if args.tool in ["graftm", "diamond"]:
-                    if not os.path.isfile(tt_obj.classification_table):
-                        # GraftM refpkg output files:
-                        gpkg_refpkg_path = os.path.join(tt_obj.refpkg_path,
-                                                        ts_evaluate.ref_pkg.prefix +
-                                                        '_' + re.sub(' ', '_', tt_obj.taxon) + ".gpkg.refpkg") + os.sep
-                        gpkg_tax_ids_file = gpkg_refpkg_path + ts_evaluate.ref_pkg.prefix + "_taxonomy.csv"
-
-                        # Copy reference files, then exclude all clades belonging to the taxon being tested
-                        output_paths = ts_clade_ex.prep_graftm_ref_files(tmp_dir=tt_obj.intermediates_dir,
-                                                                         target_clade=lineage,
-                                                                         refpkg=ts_evaluate.ref_pkg,
-                                                                         executables=ts_evaluate.executables)
-
-                        if not os.path.isdir(tt_obj.refpkg_path):
-                            ts_clade_ex.build_graftm_package(gpkg_path=tt_obj.refpkg_path,
-                                                             tax_file=output_paths["filtered_tax_ids"],
-                                                             mfa_file=output_paths["filtered_mfa"],
-                                                             fa_file=output_paths["filtered_fasta"],
-                                                             threads=args.num_threads)
-                        # Write the query sequences
-                        fasta.write_new_fasta(taxon_rep_seqs, tt_obj.test_query_fasta)
-
-                        ts_clade_ex.graftm_classify(tt_obj.test_query_fasta,
-                                                    tt_obj.refpkg_path,
-                                                    tt_obj.classifications_root,
-                                                    args.num_threads, args.tool)
-
-                        if not os.path.isfile(tt_obj.classification_table):
-                            # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
-                            logging.warning("GraftM did not generate output for " + lineage + ". Skipping.\n")
-                            shutil.rmtree(tt_obj.intermediates_dir)
-                            continue
-
-                    tt_obj.taxonomic_tree = lca_calculations.grab_graftm_taxa(gpkg_tax_ids_file)
-                    graftm_assignments = file_parsers.read_graftm_classifications(tt_obj.classification_table)
-                    tt_obj.assignments = {ts_evaluate.ref_pkg.prefix: graftm_assignments}
-                    tt_obj.filter_assignments(ts_evaluate.ref_pkg.prefix)
+                    ts_clade_ex.run_clade_exclusion_graftm(tt_obj, taxon_rep_seqs, ts_evaluate.ref_pkg,
+                                                           graftm_classifier=args.tool,
+                                                           num_threads=args.num_threads,
+                                                           executables=ts_evaluate.executables)
                 else:
-                    ce_refpkg = ts_evaluate.ref_pkg.clone(tt_obj.refpkg_path)
-
-                    if not os.path.isfile(tt_obj.classification_table):
-                        # Copy reference files, then exclude all clades belonging to the taxon being tested
-
-                        ce_refpkg.exclude_clade_from_ref_files(tt_obj.intermediates_dir, lineage,
-                                                               ts_evaluate.executables, args.fresh)
-                        # Write the query sequences
-                        fasta.write_new_fasta(taxon_rep_seqs, tt_obj.test_query_fasta)
-                        assign_args = ["-i", tt_obj.test_query_fasta, "-o", tt_obj.classifications_root,
-                                       "--refpkg_dir", os.path.dirname(ce_refpkg.f__json),
-                                       "-m", ts_evaluate.molecule_type, "-n", str(args.num_threads),
-                                       "--min_seq_length", str(ts_evaluate.min_seq_length),
-                                       "--overwrite", "--delete"]
-                        if args.trim_align:
-                            assign_args.append("--trim_align")
-                        try:
-                            assign(assign_args)
-                        except:  # Just in case treesapp assign fails, just continue
-                            pass
-
-                        if not os.path.isfile(tt_obj.classification_table):
-                            # The TaxonTest object is maintained for record-keeping (to track # queries & classifieds)
-                            logging.warning("TreeSAPP did not generate output for '{}'. Skipping.\n".format(lineage))
-                            shutil.rmtree(tt_obj.classifications_root)
-                            continue
-                    else:
-                        # Valid number of queries and these sequences have already been classified
-                        pass
-
-                    tt_obj.taxonomic_tree = ce_refpkg.all_possible_assignments()
-                    if os.path.isfile(tt_obj.classification_table):
-                        assigned_lines = file_parsers.read_classification_table(tt_obj.classification_table)
-                        tt_obj.assignments = file_parsers.parse_assignments(assigned_lines)
-                        tt_obj.filter_assignments(ts_evaluate.ref_pkg.prefix)
-                        tt_obj.distances = ts_clade_ex.parse_distances(assigned_lines)
-                    else:
-                        logging.error("marker_contig_map.tsv is missing from output directory '" +
-                                      os.path.dirname(tt_obj.classification_table) + "'\n" +
-                                      "Please remove this directory and re-run.\n")
-                        sys.exit(21)
+                    ts_clade_ex.run_clade_exclusion_treesapp(tt_obj, taxon_rep_seqs, ts_evaluate.ref_pkg,
+                                                             num_threads=args.num_threads,
+                                                             executables=ts_evaluate.executables,
+                                                             trim_align=args.trim_align,
+                                                             fresh=args.fresh,
+                                                             molecule_type=ts_evaluate.molecule_type,
+                                                             min_seq_length=ts_evaluate.min_seq_length)
+                p_bar.update()
+            p_bar.close()
 
     if ts_evaluate.stage_status("calculate"):
         # everything has been prepared, only need to parse the classifications and map lineages
@@ -1567,9 +1302,15 @@ def evaluate(sys_args):
 
                     for marker in marker_assignments:
                         ts_evaluate.markers.add(marker)
-                    rank_assignments = lca_calculations.identify_excluded_clade(marker_assignments,
-                                                                                tt_obj.taxonomic_tree,
-                                                                                ts_evaluate.ref_pkg.prefix)
+
+                    if ts_evaluate.ref_pkg.prefix not in marker_assignments:
+                        logging.debug("No sequences assigned as " + ts_evaluate.ref_pkg.prefix + "\n")
+                        rank_assignments = {}
+                    else:
+                        rank_assignments = lca_calculations.identify_excluded_clade(
+                            marker_assignments[ts_evaluate.ref_pkg.prefix],
+                            tt_obj.taxonomic_tree)
+
                     for a_rank in rank_assignments:
                         if a_rank != rank and len(rank_assignments[a_rank]) > 0:
                             logging.warning("{}-level clade excluded but optimal classification found to be {}-level.\n"
