@@ -6,6 +6,7 @@ import os
 import logging
 from time import sleep, time
 from math import ceil
+from random import randint
 
 from pyfastx import Fasta, Fastq
 from pyfastxcli import fastx_format_check
@@ -246,6 +247,7 @@ def read_fasta_to_dict(fasta_file: str) -> dict:
 
 class Header:
     def __init__(self, header):
+        """Header class for storing typical features of biological sequence headers in FASTA files"""
         self.original = header
         self.treesapp_num_id = 0
         self.post_align = ""
@@ -299,8 +301,8 @@ def register_headers(header_list: list, drop=True) -> dict:
         acc += 1
 
     if len(dups) > 0:
-        logging.warning(str(len(dups)) + " duplicate sequence headers found:\n" +
-                        ", ".join(dups) + "\n" +
+        logging.warning("{} duplicate sequence headers found:\n{}"
+                        "\n".format(len(dups), ", ".join(dups)) +
                         "TreeSAPP will proceed as if these duplicate sequences were never seen in 5 seconds or"
                         " you can stop it here with Ctrl-c... the choice is yours.\n")
         sleep(5)
@@ -310,6 +312,7 @@ def register_headers(header_list: list, drop=True) -> dict:
 
 class FASTA:
     def __init__(self, file_name):
+        """Instantiate a new class instance for storing and interacting with FASTA files"""
         self.file = file_name
         self.fasta_dict = dict()
         self.header_registry = dict()  # A dictionary of Header instances indexed by a unique numerical identifier
@@ -330,12 +333,24 @@ class FASTA:
         self.index_form = fasta.index_form
         return
 
-    def load_fasta(self):
-        self.fasta_dict = read_fasta_to_dict(self.file)
+    def clear(self):
+        self.file = ""
+        self.fasta_dict.clear()
+        self.header_registry.clear()
+        self.amendments.clear()
+        self.index_form = None
+        return
+
+    def load_fasta(self, format_it=False, molecule="prot") -> None:
+        if format_it:
+            self.fasta_dict = format_read_fasta(self.file, molecule)
+        else:
+            self.fasta_dict = read_fasta_to_dict(self.file)
         self.header_registry = register_headers(get_headers(self.file), True)
         if len(self.fasta_dict) == 0 and len(self.header_registry) == 0:
             logging.error("FASTA file '{}' is empty or corrupted - no sequences were found!\n".format(self.file))
             sys.exit(3)
+        return
 
     def add_accession_to_headers(self, refpkg_name=""):
         for acc in self.header_registry:
@@ -356,10 +371,10 @@ class FASTA:
             logging.warning("FASTA header registry and fasta dictionary are out of sync!\n")
         return len(self.fasta_dict.keys())
 
-    def original_header_map(self):
+    def original_header_map(self) -> dict:
         return {self.header_registry[index].original: [self.header_registry[index]] for index in self.header_registry}
 
-    def first_split_header_map(self):
+    def first_split_header_map(self) -> dict:
         header_map = dict()
         for header in self.header_registry.values():  # type: Header
             fs_h = header.first_split
@@ -452,8 +467,17 @@ class FASTA:
         logging.debug("Identified and replaced invalid ambiguity characters in %d sequences.\n" % bad_seqs)
 
         return
+    
+    def trim_to_length(self, seq_length: int):
+        self.remove_shorter_than(seq_length)
+        for seq_id in self.fasta_dict:
+            max_stop = len(self.fasta_dict[seq_id]) - seq_length
+            random_start = randint(0, max_stop)
+            self.fasta_dict[seq_id] = self.fasta_dict[seq_id][random_start:random_start + seq_length]
 
-    def remove_shorter_than(self, min_len: int):
+        return
+
+    def remove_shorter_than(self, min_len: int) -> int:
         long_seqs = list()
         dropped = 0
         for seq_name in self.fasta_dict:
@@ -461,10 +485,15 @@ class FASTA:
                 long_seqs.append(seq_name)
             else:
                 dropped += 1
-        if dropped >= 1:
+        if dropped == self.n_seqs():
+            logging.debug("All {} sequences were shorter than the minimum sequence length threshold ({}).\n"
+                          "".format(dropped, min_len))
+            self.clear()
+            return 1
+        elif dropped >= 1:
             logging.debug("{} sequences were found to be shorter than {} and removed.\n".format(dropped, min_len))
         self.keep_only(long_seqs)
-        return
+        return 0
 
     def get_header_mapping_dict(self) -> dict:
         """
@@ -543,12 +572,14 @@ class FASTA:
                     excluded_headers.append(self.header_registry[num_id].original)
                 except KeyError:
                     logging.error("Unable to find TreeSAPP ID '%s' in header_registry.\n" % num_id)
-                    sys.exit()
+                    sys.exit(3)
             self.header_registry = sync_header_registry
             self.fasta_dict = sync_fasta_dict
+
+        # Writing the individual headers leads to massive log files...
         if len(excluded_headers) >= 1:
-            logging.debug("The following sequences were excluded after synchronizing FASTA:\n\t" +
-                          "\n\t".join(excluded_headers) + "\n")
+            logging.debug("{} sequences were excluded after synchronizing FASTA.\n".format(len(excluded_headers)))
+
         if len(self.header_registry) == 0:
             logging.error("All sequences were discarded during header_registry and fasta_dict synchronization.\n")
             sys.exit(-1)
@@ -741,7 +772,7 @@ def merge_fasta_dicts_by_index(extracted_seq_dict, numeric_contig_index):
     return merged_extracted_seq_dict
 
 
-def format_fasta(fasta_input: str, molecule: str, output_fasta: str, min_seq_length=10) -> dict:
+def format_fasta(fasta_input: str, molecule: str, output_fasta: str, min_seq_length=10, full_name=True) -> dict:
     """
     Reads a FASTA file, ensuring each sequence and sequence name is valid, and writes the valid sequence to a new FASTA.
     Only headers are read into memory and the formatted FASTA is saved to a buffer before written to a file and cleared.
@@ -750,6 +781,7 @@ def format_fasta(fasta_input: str, molecule: str, output_fasta: str, min_seq_len
     :param molecule: Molecule type of the sequences ['prot', 'dna', 'rrna']
     :param output_fasta: Path to the formatted FASTA file to write
     :param min_seq_length: All sequences shorter than this will not be included in the returned list.
+    :param full_name: Controls whether the formatted FASTA file has the full name (True) or first name (False)
     :return: A dictionary of Header instances indexed by a numerical identifier
     """
     start = time()
@@ -772,7 +804,7 @@ def format_fasta(fasta_input: str, molecule: str, output_fasta: str, min_seq_len
     max_buffer_size = 1E4
     seq_acc = 0
     fasta_string = ""
-    for name, seq in Fasta(fasta_input, build_index=False, full_name=True):  # type: (str, str)
+    for name, seq in Fasta(fasta_input, build_index=False, full_name=full_name):  # type: (str, str)
         if len(seq) < min_seq_length:
             continue
         if bad_chars.search(seq):
@@ -805,15 +837,11 @@ def format_fasta(fasta_input: str, molecule: str, output_fasta: str, min_seq_len
                       "\n".join(bad_seqs) + "\n")
 
     header_registry = register_headers(headers, True)
-    # if len(header_registry) != seq_acc:
-    #     logging.error("The number of sequences read ({}) does not equal"
-    #                   " the number of sequence names registered ({}).\n".format(seq_acc, len(header_registry)))
-    #     sys.exit(13)
 
     return header_registry
 
 
-def format_read_fasta(fasta_input: str, molecule: str, subset=None, min_seq_length=10):
+def format_read_fasta(fasta_input: str, molecule: str, subset=None, min_seq_length=10, full_name=True):
     """
     Reads a FASTA file, ensuring each sequence and sequence name is valid.
 
@@ -822,6 +850,7 @@ def format_read_fasta(fasta_input: str, molecule: str, subset=None, min_seq_leng
     :param subset: A set for filtering sequences. Only sequences with names in subset will be included in the dictionary
     :type subset: set
     :param min_seq_length: All sequences shorter than this will not be included in the returned list.
+    :param full_name: Controls whether the formatted FASTA file has the full name (True) or first name (False)
     :return: A Python dictionary with headers as keys and sequences as values
     """
     start = time()
@@ -838,7 +867,7 @@ def format_read_fasta(fasta_input: str, molecule: str, subset=None, min_seq_leng
         sys.exit(13)
 
     try:
-        py_fa = Fasta(fasta_input, build_index=False, full_name=True)
+        py_fa = Fasta(fasta_input, build_index=False, full_name=full_name)
     except RuntimeError as error:
         logging.debug(str(error)+"\n")
         return formatted_fasta_dict
