@@ -8,16 +8,16 @@ import sys
 import logging
 from time import sleep
 from glob import glob
+
 from numpy import sqrt
 
-from ete3 import Tree
-
 from treesapp import file_parsers
-from treesapp.phylo_seq import assignments_to_treesaps
+from treesapp import wrapper
+from treesapp import utilities
+from treesapp import assign
+from treesapp.phylo_seq import assignments_to_pqueries
 from treesapp.refpkg import ReferencePackage
-from treesapp.commands import assign
 from treesapp.fasta import get_headers, register_headers
-from treesapp.external_command_interface import launch_write_command
 from treesapp.classy import prep_logging, get_header_info
 from treesapp.entrez_utils import EntrezRecord, get_multiple_lineages
 from treesapp import lca_calculations as ts_lca
@@ -33,6 +33,7 @@ class ConfusionTest:
         self.fn = {key: [] for key in gene_list}
         self.fp = {key: set() for key in gene_list}
         self.tp = {key: [] for key in gene_list}  # This will be a list of QuerySequence instances
+        self.unlabelled_tp_query_names = {}  # These are to be added to the table at distance seven
         self.redundant_queries = set()
         self.tax_lineage_map = {}
         self.tp_lineage_map = {}
@@ -103,36 +104,6 @@ class ConfusionTest:
         summary_string += "\tTrue negatives\t\t" + str(self.get_true_negatives(refpkg_name)) + "\n"
         return summary_string
 
-    def populate_tax_lineage_map(self, entrez_record_dict: dict) -> None:
-        """
-        Used for pulling the NCBI taxid from a sequence name (header). This is most effective for handling
-        EggNOG-formatted headers where the NCBI taxid is the first part, followed by a period, then the organism name
-
-        :param entrez_record_dict: A dictionary mapping unique numerical TreeSAPP identifiers to EntrezRecord instances
-        :return: None
-        """
-        for ts_id in entrez_record_dict:  # type: str
-            e_record = entrez_record_dict[ts_id]  # type: EntrezRecord
-            if not e_record.description:
-                logging.error("No description for the following EntrezRecord:\n{}\n".format(e_record.get_info()))
-                sys.exit(3)
-
-            # Only make Entrez records for new NCBI taxonomy IDs
-            if e_record.description not in self.tax_lineage_map:
-                if not e_record.lineage:
-                    logging.warning("Lineage information unavailable for accession '{}'\n".format(e_record.accession))
-                    self.tax_lineage_map[e_record.description] = ''
-                else:
-                    self.tax_lineage_map[e_record.description] = e_record.lineage
-
-        return
-
-    def populate_true_positive_lineage_map(self) -> None:
-        for refpkg_code in self.ref_packages:
-            self.tp_lineage_map[refpkg_code] = {tp_inst.ref_name: tp_inst.true_lineage
-                                                for tp_inst in self.tp[refpkg_code]}
-        return
-
     def generate_entrez_queries(self) -> None:
         entrez_record_dict = get_header_info(self.header_registry)
         for index, e_record in entrez_record_dict.items():  # type: EntrezRecord
@@ -149,16 +120,26 @@ class ConfusionTest:
         """
         # Gather the unique taxonomy IDs and store in EntrezRecord instances
         t_hierarchy = TaxonomicHierarchy()
+        unknowns = {}
         entrez_records = [self.entrez_query_dict[index] for index in self.entrez_query_dict]
         # Query the Entrez database for these unique taxonomy IDs
         get_multiple_lineages(entrez_records, t_hierarchy, "prot")
 
         for e_record in entrez_records:  # type: EntrezRecord
             if not e_record.lineage:
-                logging.warning("Lineage information unavailable for taxonomy ID '" + e_record.ncbi_tax + "'\n")
+                try:
+                    unknowns[e_record.ncbi_tax] += 1
+                except KeyError:
+                    unknowns[e_record.ncbi_tax] = 1
                 self.tax_lineage_map[e_record.ncbi_tax] = ''
             else:
                 self.tax_lineage_map[e_record.ncbi_tax] = "r__Root; " + e_record.lineage
+
+        if unknowns:
+            warn_str = ""
+            for taxid in unknowns:
+                warn_str += "\t{} query sequences with unknown taxid '{}'\n".format(unknowns[taxid], taxid)
+            logging.warning("Lineage information unavailable for taxonomy IDs:\n" + warn_str)
         return
 
     def map_true_lineages(self) -> None:
@@ -178,6 +159,9 @@ class ConfusionTest:
                 info_string += "\t".join([marker, tp_inst.place_name, str(tp_inst.tax_dist), "True",
                                           tp_inst.true_lineage, tp_inst.assigned_lineage,
                                           tp_inst.optimal_lineage]) + "\n"
+            if marker in self.unlabelled_tp_query_names:
+                for pquery in self.unlabelled_tp_query_names[marker]:
+                    info_string += "\t".join([marker, pquery.seq_name, "NA", "True", "NA", "NA", "NA"]) + "\n"
         for marker in self.fn:
             for tp_inst in self.fn[marker]:  # type: QuerySequence
                 # TODO: Support headers from databases other than EggNOG
@@ -237,36 +221,6 @@ class ConfusionTest:
                     self.dist_wise_tp[marker][tp_inst.tax_dist] = [tp_inst.place_name]
         return
 
-    def enumerate_optimal_rank_distances(self) -> dict:
-        """
-        Calculates the number of query sequences representing an excluded rank.
-
-        For each query sequence in self.tp (so just the true positives), the optimal taxonomy is identified based on
-        the query's true taxonomic lineage and all the taxonomic lineages in the ReferencePackage.taxa_trie.
-        The optimal taxonomic assignment's depth (i.e. numerical description of a rank where domain = 1,
-         phylum = 2, etc.) is found by determining the length of the list after splitting the lineage string by '; '.
-
-        :return: Dictionary mapping the numerical description of a taxonomic rank (i.e. depth) to the number of queries
-        that represent that rank.
-        """
-        rank_representation = dict()
-        for marker in self.tp:
-            refpkg = self.ref_packages[marker]  # type: ReferencePackage
-            refpkg.taxa_trie.trie_check()
-
-            for tp_inst in self.tp[marker]:  # type: QuerySequence
-                # Find the optimal taxonomic assignment
-                optimal_taxon = ts_lca.optimal_taxonomic_assignment(refpkg.taxa_trie.trie, tp_inst.true_lineage)
-                if not optimal_taxon:
-                    logging.debug("Optimal taxonomic assignment '{}' for {} "
-                                  "not found in reference hierarchy.\n".format(tp_inst.true_lineage, tp_inst.place_name))
-                    continue
-                try:
-                    rank_representation[len(optimal_taxon.split("; "))] += 1
-                except KeyError:
-                    rank_representation[len(optimal_taxon.split("; "))] = 1
-        return rank_representation
-
     def check_dist(self):
         if self._MAX_TAX_DIST < 0:
             logging.error("ConfusionTest's _MAX_TAX_DIST has yet to be set.\n")
@@ -335,7 +289,32 @@ class ConfusionTest:
                 unique_fn.update(qseq.place_name for qseq in self.fn[marker])
             return unique_fn
 
+    def gather_unlabelled_queries(self, assignments: dict, positives: set) -> None:
+        """
+        Finds all query sequences that were not assigned a lineage by Entrez.
+        This is not uncommon and typically caused by NCBI merging taxid labels.
+
+        :param assignments: A dictionary mapping ReferencePackage.prefix to a list of PQuery instances that were
+         assigned to that reference package.
+        :param positives: Names of the positive class query sequences
+        :return: A list of all PQuery instances that were not assigned a lineage from self.entrez_query_dict
+        """
+        for refpkg_name, pqueries in assignments.items():  # type: (str, list)
+            for pquery in sorted(pqueries, key=lambda x: x.seq_name):
+                try:
+                    e_record = self.entrez_query_dict[pquery.seq_name]
+                except KeyError:
+                    continue
+                if not e_record.lineage:
+                    if pquery.seq_name in positives:
+                        try:
+                            self.unlabelled_tp_query_names[refpkg_name].append(pquery)
+                        except KeyError:
+                            self.unlabelled_tp_query_names[refpkg_name] = [pquery]
+        return
+
     def bin_headers(self, assignments: dict, annot_map: dict) -> None:
+        self.gather_unlabelled_queries(assignments, set(annot_map.keys()))
         binned_tp, binned_fp, binned_fn = bin_headers(assignments, annot_map, self.entrez_query_dict, self.ref_packages)
 
         self.tp.update(binned_tp)
@@ -357,9 +336,11 @@ class ConfusionTest:
                 seq_name = qseq.place_name
                 if seq_name[0] == '>':
                     seq_name = seq_name[1:]
-                if seq_name not in tp_names:
+                if seq_name not in tp_names and seq_name in self.all_queries:
                     validated_fp.add(qseq)
-            self.fp[marker] = self.all_queries.intersection(validated_fp)
+
+            # Ensure the FP queries are in the set of unique queries from the input fasta
+            self.fp[marker] = validated_fp
         return
 
     def validate_false_negatives(self, refpkg_dbname_dict: dict):
@@ -372,7 +353,8 @@ class ConfusionTest:
                     refpkg_og_map[og].append(refpkg_name)
                 except KeyError:
                     refpkg_og_map[og] = [refpkg_name]
-
+        fn_names = set()
+        validated_fns = set()
         for marker in self.fn:
             if marker in refpkg_dbname_dict:
                 homologous_tp_names = set()
@@ -380,12 +362,18 @@ class ConfusionTest:
                     for homolgous_marker in refpkg_og_map[og]:
                         if homolgous_marker != marker:
                             homologous_tp_names.update(set(qs.place_name for qs in self.tp[homolgous_marker]))
-                validated_fns = set()
+
                 for qseq in self.fn[marker]:  # type: QuerySequence
                     if qseq.place_name not in homologous_tp_names:
-                        validated_fns.add(qseq)
+                        fn_names.add(qseq.place_name)
                 # Remove all sequences from this marker's false negatives that are found in a homologous TP set
-                self.fn[marker] = self.all_queries.intersection(validated_fns)
+                validated_fn_names = self.all_queries.intersection(fn_names)
+                for qseq in self.fn[marker]:
+                    if qseq.place_name in validated_fn_names:
+                        validated_fns.add(qseq)
+                self.fn[marker] = self.fn[marker].intersection(validated_fns)
+                fn_names.clear()
+                validated_fns.clear()
         return
 
     def summarise_type_one_placements(self, classification_lines) -> str:
@@ -416,7 +404,7 @@ class ConfusionTest:
             if header in self.fp[refpkg_map[refpkg_name]]:
                 distal, pendant, avg = [round(float(x), 3) for x in dists.split(',')]
                 # hmm_perc = round((int(length)*100)/hmm_lengths[refpkg], 0)
-                descendents = len(internal_nodes_dict[refpkg_name][i_node])
+                descendents = len(internal_nodes_dict[refpkg_name][int(i_node)])
                 if descendents not in summary_dict["leaves"]:
                     summary_dict["leaves"][descendents] = 0
                 summary_dict["leaves"][descendents] += 1
@@ -483,6 +471,31 @@ class ConfusionTest:
 
         return summary_string
 
+    def tabularise_mcc_stats(self, mcc_table_file: str) -> None:
+        d = 0
+        mcc_string = "Tax.dist\tMCC\tTrue.Pos\tTrue.Neg\tFalse.Pos\tFalse.Neg\n"
+        while d < 8:
+            self._MAX_TAX_DIST = d
+            tp, remainder = self.get_true_positives_at_dist()
+            num_tp = len(tp)
+            num_fp = len(self.get_false_positives()) + len(remainder)
+            num_fn = len(self.get_false_negatives())
+            num_tn = self.get_true_negatives()
+
+            # Use the query sequences lacking taxonomic lineage labels at this point
+            if d == 7:
+                n_unlabelled = sum([len(rp_queries) for rp_queries in self.unlabelled_tp_query_names.values()])
+                num_tp += n_unlabelled
+                num_tn -= n_unlabelled
+
+            mcc = calculate_matthews_correlation_coefficient(num_tp, num_fp, num_fn, num_tn)
+            mcc_string += "\t".join([str(x) for x in [d, mcc, num_tp, num_tn, num_fp, num_fn]]) + "\n"
+            d += 1
+        logging.info(mcc_string)
+        with open(mcc_table_file, 'w') as mcc_handler:
+            mcc_handler.write(mcc_string)
+        return
+
 
 def map_lineages(qseq_collection: set, tax_lineage_map: dict) -> None:
     for qseq in qseq_collection:  # type: QuerySequence
@@ -580,24 +593,6 @@ def get_arguments(sys_args):
     return args
 
 
-def internal_node_leaf_map(tree: str) -> dict:
-    """
-    Loads a Newick-formatted tree with internal nodes into a dictionary of
-    all internal nodes (keys) and a list of child leaves (values).
-
-    :param tree: Path to a Newick tree file.
-    :return: Dictionary of all internal nodes (keys) and a list of child leaves (values)
-    """
-    node_map = dict()
-    x = 0
-    ete_tree = Tree(tree)
-    for node in ete_tree.traverse(strategy="postorder"):
-        node_map[str(x)] = node.get_leaf_names()
-        x += 1
-
-    return node_map
-
-
 def validate_command(args, sys_args):
     logging.debug("Command used:\n" + ' '.join(sys_args) + "\n")
 
@@ -637,7 +632,7 @@ def calculate_matthews_correlation_coefficient(tp: int, fp: int, fn: int, tn: in
     numerator = float((tp * tn) - (fp * fn))
     denominator = float((tp + fp) * (tp + fn) * (tn + fp) * (tn + fn))
     if numerator == 0 or denominator == 0:
-        return 0
+        return 0.0
     else:
         return round(numerator/sqrt(denominator), 3)
 
@@ -764,6 +759,7 @@ def mcc_calculator(sys_args):
     mcc_file = output_prefix + "_MCC_table.tsv"
     taxa_dist_output = output_prefix + '_' + summary_rank + "_dist.tsv"
     classification_info_output = output_prefix + "_classifications.tsv"
+    graftm_exe = ""
 
     # Instantiate the logging instance and write the log
     prep_logging(log_name, args.verbose)
@@ -804,6 +800,7 @@ def mcc_calculator(sys_args):
     if args.tool == "treesapp":
         test_obj.ref_packages = refpkg_dict
     elif args.tool in ["graftm", "diamond"]:
+        graftm_exe = utilities.fetch_executable_path("graftM", test_obj.treesapp_dir)
         for gpkg in glob(args.refpkg_dir + "*gpkg"):
             gpkg_name = str(os.path.basename(gpkg).split('.')[0])
             if gpkg_name in test_obj.ref_packages:
@@ -835,14 +832,15 @@ def mcc_calculator(sys_args):
                              "--stringency", args.stringency,
                              "--placement_summary", args.p_sum,
                              "--min_like_weight_ratio", str(args.min_lwr),
+                             "--max_pendant_length", str(args.max_pd),
                              "--overwrite", "--delete"]
             if args.trim_align:
                 classify_args.append("--trim_align")
             if args.svm:
                 classify_args.append("--svm")
-            assign(classify_args)
+            assign.assign(classify_args)
         classification_lines = file_parsers.read_classification_table(classification_table)
-        assignments = assignments_to_treesaps(classification_lines)
+        assignments = assignments_to_pqueries(classification_lines)
     else:
         # Since you are only able to analyze a single reference package at a time with GraftM, this is ran iteratively
         for gpkg in glob(args.refpkg_dir + "*gpkg"):
@@ -854,21 +852,10 @@ def mcc_calculator(sys_args):
                 logging.warning("'{}' not in {} and will be skipped...\n".format(pkg_name, args.annot_map))
                 continue
             output_dir = test_obj.data_dir + pkg_name + os.sep
-            if not os.path.isdir(output_dir):
-                os.makedirs(output_dir)
             classification_table = output_dir + test_fa_prefix + os.sep + test_fa_prefix + "_read_tax.tsv"
             if not os.path.isfile(classification_table):
-                classify_call = ["graftM", "graft",
-                                 "--forward", args.input,
-                                 "--graftm_package", gpkg,
-                                 "--input_sequence_type", "aminoacid",
-                                 "--threads", str(args.num_threads),
-                                 "--output_directory", output_dir,
-                                 "--force"]
-                if args.tool == "diamond":
-                    classify_call += ["--assignment_method", "diamond"]
-                    classify_call += ["--search_method", "diamond"]
-                launch_write_command(classify_call, False)
+                wrapper.run_graftm_graft(graftm_exe, args.input, output_dir,
+                                         gpkg_path=gpkg, classifier=args.tool, num_threads=args.num_threads)
 
             # TODO: Figure out how to convert GraftM classifications into JPlace objects
             assignments[pkg_name] = file_parsers.read_graftm_classifications(classification_table)
@@ -903,21 +890,7 @@ def mcc_calculator(sys_args):
     ##
     test_obj._MAX_TAX_DIST = 6
     logging.debug(test_obj.get_info(True))
-    d = 0
-    mcc_string = "Tax.dist\tMCC\tTrue.Pos\tTrue.Neg\tFalse.Pos\tFalse.Neg\n"
-    while d < 8:
-        test_obj._MAX_TAX_DIST = d
-        tp, remainder = test_obj.get_true_positives_at_dist()
-        num_tp = len(tp)
-        num_fp = len(test_obj.get_false_positives()) + len(remainder)
-        num_fn = len(test_obj.get_false_negatives())
-        num_tn = test_obj.get_true_negatives()
-        mcc = calculate_matthews_correlation_coefficient(num_tp, num_fp, num_fn, num_tn)
-        mcc_string += "\t".join([str(x) for x in [d, mcc, num_tp, num_tn, num_fp, num_fn]]) + "\n"
-        d += 1
-    logging.info(mcc_string)
-    with open(mcc_file, 'w') as mcc_handler:
-        mcc_handler.write(mcc_string)
+    test_obj.tabularise_mcc_stats(mcc_file)
     return
 
 
