@@ -237,12 +237,12 @@ class PQuery:
             self.classified = False
         return
 
-    def sum_rpkms_per_node(self, leaf_rpkm_sums: dict) -> dict:
+    def sum_abundances_per_node(self, leaf_abundance_sums: dict) -> dict:
         """
-        Function that adds the RPKM value of a contig to the node it was placed.
-        For contigs mapping to internal nodes: the proportional RPKM assigned is summed for all children.
+        Function that adds the abundance value of a contig to the node it was placed.
+        For contigs mapping to internal nodes: the proportional abundance assigned is summed for all children.
 
-        :param leaf_rpkm_sums: A dictionary mapping tree leaf numbers to abundances (RPKM sums)
+        :param leaf_abundance_sums: A dictionary mapping tree leaf numbers to abundances (RPKM sums)
         :return: A dictionary mapping numerical leaf node identifiers to normalized abundance values
         """
         jplace_node = self.consensus_placement.edge_num
@@ -254,10 +254,10 @@ class PQuery:
             normalized_abundance = 0.0
 
         for tree_leaf in tree_leaves:  # type: str
-            if tree_leaf not in leaf_rpkm_sums.keys():
-                leaf_rpkm_sums[tree_leaf] = 0.0
-            leaf_rpkm_sums[tree_leaf] += normalized_abundance
-        return leaf_rpkm_sums
+            if tree_leaf not in leaf_abundance_sums.keys():
+                leaf_abundance_sums[tree_leaf] = 0.0
+            leaf_abundance_sums[tree_leaf] += normalized_abundance
+        return leaf_abundance_sums
 
     def check_jplace_edge_lengths(self, tree_index: dict) -> None:
         """
@@ -295,6 +295,9 @@ class PQuery:
             logging.error("Unable to find placement edge '{}'"
                           " in the PhyloPlace's node_map.\n".format(self.consensus_placement.edge_num))
             sys.exit(13)
+        except AttributeError:
+            logging.error("Pquery.consensus_placement was not instantiated.\n")
+            sys.exit(15)
 
         for leaf_node in tree_leaves:
             try:
@@ -456,7 +459,7 @@ class PQuery:
         return
 
 
-def assignments_to_treesaps(classified_lines: list) -> dict:
+def assignments_to_pqueries(classified_lines: list) -> dict:
     """
     Used for converting the TreeSAPP-assignment information of classified sequences (found in self.classifications)
     into JPlace instances such that these can be reproducibly modified and written again, if needed.
@@ -509,6 +512,33 @@ def split_placements(placements: dict) -> list:
     return phylo_places
 
 
+def abundify_tree_saps(tree_saps: dict, abundance_dict: dict):
+    """
+    Add abundance (RPKM or presence count) values to the PQuery instances (abundance variable)
+
+    :param tree_saps: Dictionary mapping refpkg codes to all PQuery instances for classified sequences
+    :param abundance_dict: Dictionary mapping sequence names to floats
+    :return: None
+    """
+    abundance_mapped_acc = 0
+    for placed_seqs in tree_saps.values():  # type: list
+        for pquery in placed_seqs:  # type: PQuery
+            if not pquery.abundance:
+                # Filter out RPKMs for contigs not associated with the target marker
+                try:
+                    pquery.abundance = abundance_dict[pquery.place_name]
+                    abundance_mapped_acc += 1
+                except KeyError:
+                    pquery.abundance = 0.0
+            else:
+                abundance_mapped_acc += 1
+
+    if abundance_mapped_acc == 0:
+        logging.warning("No placed sequences with abundances identified.\n")
+
+    return
+
+
 class TreeLeafReference:
     """
     Objects for each leaf in a tree
