@@ -125,8 +125,11 @@ class ReferencePackage:
             self.bail("ReferencePackage.f__json not set. ReferencePackage band() cannot be completed.\n")
             raise AttributeError
 
-        if not os.path.isdir(os.path.dirname(self.f__json)):
-            os.mkdir(os.path.dirname(self.f__json))
+        output_dir = os.path.dirname(self.f__json)
+        if not output_dir:
+            output_dir = "./"
+        if not os.path.isdir(output_dir):
+            os.mkdir(output_dir)
 
         try:
             refpkg_handler = open(self.f__json, 'wb')
@@ -240,7 +243,7 @@ class ReferencePackage:
         self.write_refpkg_component(self.f__model_info, self.model_info)
         self.write_refpkg_component(self.f__tree, self.tree)
         if len(self.boot_tree) > 0:
-            self.write_refpkg_component(output_prefix + self.f__boot_tree, self.boot_tree)
+            self.write_refpkg_component(self.f__boot_tree, self.boot_tree)
 
         return
 
@@ -441,6 +444,8 @@ class ReferencePackage:
         for treesapp_id in sorted(self.lineage_ids, key=int):
             seq_name, lineage = self.lineage_ids[treesapp_id].split("\t")
             ref = TreeLeafReference(treesapp_id, seq_name)
+            if self.taxa_trie.root_taxon != lineage.split(self.taxa_trie.lin_sep)[0]:
+                lineage = self.taxa_trie.root_taxon + self.taxa_trie.lin_sep + lineage
             ref.lineage = lineage
             ref_leaf_nodes.append(ref)
         return ref_leaf_nodes
@@ -728,7 +733,7 @@ class ReferencePackage:
 
     def recover_raxmlng_tree_outputs(self, phylogeny_dir: str) -> None:
         # Find the best tree file
-        raw_newick_tree = match_file(phylogeny_dir + "*.bestTree")
+        raw_newick_tree = match_file("{}{}.{}.nwk".format(phylogeny_dir, self.prefix, self.tree_tool))
 
         # Import the tree files into the reference package
         copy(raw_newick_tree, self.f__tree)
@@ -953,6 +958,26 @@ class ReferencePackage:
 
         return load_taxonomic_trie(lineage_list)
 
+    @staticmethod
+    def propagate_internal_node_taxon_labels_from_leaves(ref_tree: Tree) -> None:
+        # Label the internal nodes based on the LCA of the leaf nodes
+        for n in ref_tree.traverse(strategy="postorder"):
+            if len(n.children) == 0:  # this is a leaf node
+                continue
+            elif len(n.children) == 1:
+                logging.error("Unexpected number of children (1) for Tree.Node {}.\n".format(n.name))
+                raise AssertionError
+            elif len(n.children) > 2:
+                n.resolve_polytomy(recursive=False)
+                for c in n.children:
+                    if not hasattr(c, "taxon"):
+                        c.add_feature(pr_name="taxon", pr_value=None)
+                        c.taxon = Taxon.lca(c.children[0].taxon, c.children[1].taxon)
+            l_node, r_node = n.get_children()
+            lca = Taxon.lca(l_node.taxon, r_node.taxon)
+            n.taxon = lca
+        return
+
     def taxonomically_label_tree(self) -> Tree:
         """
         When deciding what the taxonomic label should be assigned to a query sequence, algorithms may require a tree
@@ -978,22 +1003,7 @@ class ReferencePackage:
             tree_node = rt.get_leaves_by_name(name="{}_{}".format(leaf_node.number, self.prefix)).pop()
             tree_node.add_feature(pr_name="taxon", pr_value=taxon)
 
-        # Label the internal nodes based on the LCA of the leaf nodes
-        for n in rt.traverse(strategy="postorder"):
-            if len(n.children) == 0:  # this is a leaf node
-                continue
-            elif len(n.children) == 1:
-                self.bail("Unexpected number of children (1) for Tree.Node {}.\n".format(n.name))
-                raise AssertionError
-            elif len(n.children) > 2:
-                n.resolve_polytomy(recursive=False)
-                for c in n.children:
-                    if not hasattr(c, "taxon"):
-                        c.add_feature(pr_name="taxon", pr_value=None)
-                        c.taxon = Taxon.lca(c.children[0].taxon, c.children[1].taxon)
-            l_node, r_node = n.get_children()
-            lca = Taxon.lca(l_node.taxon, r_node.taxon)
-            n.taxon = lca
+        self.propagate_internal_node_taxon_labels_from_leaves(rt)
 
         return rt
 
@@ -1018,11 +1028,12 @@ def view(refpkg: ReferencePackage, attributes: list) -> None:
             sys.exit(1)
 
     for k, v in view_dict.items():
+        logging.info("{}:\t".format(k))
         if type(v) is list and k not in ["pfit"]:
             v = ''.join(v)
         if type(v) is dict:
             v = "\n" + "\n".join([sk + "\t" + sv for sk, sv in v.items()])
-        logging.info("{}\t{}\n".format(k, v))
+        sys.stdout.write(str(v).strip() + "\n")
 
     # TODO: optionally use ReferencePackage.write_refpkg_component
     return
@@ -1077,8 +1088,7 @@ def rename(refpkg: ReferencePackage, attributes: list, output_dir: str, overwrit
         sys.exit(3)
 
     new_prefix = attributes.pop()
-    edit_str_attrs = ["f__json",
-                      "f__msa",
+    edit_str_attrs = ["f__msa",
                       "f__profile", "f__search_profile",
                       "f__tree", "f__boot_tree", "f__model_info", "tree"]
     edit_iter_attrs = ["msa", "profile", "search_profile"]
@@ -1095,6 +1105,7 @@ def rename(refpkg: ReferencePackage, attributes: list, output_dir: str, overwrit
 
     # Finally change the value of the prefix once all other attributes have been modified
     refpkg.prefix = new_prefix
+    refpkg.f__json = refpkg.prefix + refpkg.refpkg_suffix
 
     write_edited_pkl(refpkg, output_dir, overwrite)
 
