@@ -76,6 +76,12 @@ class Taxon:
             return False
         return True
 
+    def get_rank_in_lineage(self, rank_name: str):
+        for t in self.lineage():
+            if t.rank == rank_name:
+                return t
+        return
+
     @staticmethod
     def lca(left_taxon, right_taxon):
         try:
@@ -127,7 +133,7 @@ class TaxonomicHierarchy:
         self.taxon_sep = "__"  # Separator between the rank prefix and the taxon name
         self.root_taxon = "r" + self.taxon_sep + "Root"
         self.bad_taxa = ["cellular organisms", "unclassified"]  # Optional list that can be used to remove some taxa
-        self.canonical_prefix = re.compile(r"^[nrdpcofgs]" + re.escape(self.taxon_sep))
+        self.canonical_prefix = re.compile(r"^[nrdpcofgst]" + re.escape(self.taxon_sep))
         self.proper_species_re = re.compile("^(s__)?[A-Z][a-z]+ [a-z]+$")
         self.no_rank_re = re.compile(r"^" + re.escape(self.no_rank_name[0] + self.taxon_sep) + r".*")
         self.conflicts = set()  # A tuple to store Taxon instances that create conflicting paths in the hierarchy
@@ -205,6 +211,18 @@ class TaxonomicHierarchy:
             logging.debug("Taxon name '{}' not present in taxonomic hierarchy.\n".format(prefix_taxon))
             return
 
+    def get_taxon_descendents(self, prefix_taxon: str) -> list:
+        """Returns a list of all Taxon instances with prefix_taxon in their lineage."""
+        descendents = []
+        ref_taxon = self.get_taxon(prefix_taxon)
+        if not ref_taxon:
+            logging.warning("No descendents of '{}' found as it's not in the hierarchy.\n".format(prefix_taxon))
+            return descendents
+        for hierarchy_taxon in self.hierarchy.values():  # type: Taxon
+            if ref_taxon in hierarchy_taxon.lineage():
+                descendents.append(hierarchy_taxon)
+        return descendents
+
     def get_bare_taxon(self, taxon_name: str):
         """
         A function for querying the hierarchy with a taxon that lacks a rank-prefix.
@@ -417,7 +435,7 @@ class TaxonomicHierarchy:
             return replaced_nodes
 
         self.conflicts = self.order_conflict_taxa()
-        conflict_resolution_summary = "Taxonomic hierarchy conflicts were resolved by merging the left taxon into the right:\n"
+        conflict_resolution_summary = "Taxonomic hierarchy conflicts were fixed by merging the left into the right:\n"
         while self.conflicts:
             node_one, node_two = self.conflicts.pop(0)  # type: (Taxon, Taxon)
 
@@ -453,7 +471,7 @@ class TaxonomicHierarchy:
                                                                                rep.name, rep.rank)
 
         logging.debug(conflict_resolution_summary)
-        self.conflicts = set()
+        self.conflicts = set(self.conflicts)
         return replaced_nodes
 
     def validate_rank_prefixes(self) -> None:
@@ -663,7 +681,12 @@ class TaxonomicHierarchy:
                               "taxon = {0}\n"
                               "taxon_info = {1}\n".format(taxon_name, str(taxon_info)))
                 sys.exit(11)
-            rank = taxon_info["Rank"]
+            try:
+                rank = taxon_info["Rank"]
+            except KeyError:
+                logging.warning("'Rank' attribute missing from Entrez record '{}'. "
+                                "Rank set to '{}'.\n".format(taxon_info, self.no_rank_name))
+                rank = self.no_rank_name
 
             taxon = self.digest_taxon(taxon_name, rank, previous=previous)  # type: Taxon
             if not taxon and previous:
@@ -732,10 +755,18 @@ class TaxonomicHierarchy:
         self.validate_rank_prefixes()
 
         previous = None
+        no_domain = []
         for ref_leaf in ref_leaves:  # type: TreeLeafReference
             if not ref_leaf.lineage:
                 continue
             taxa = ref_leaf.lineage.split(self.lin_sep)
+            if taxa[0] != self.root_taxon:
+                # Look for domain
+                if 'd' not in set([taxon.split(self.taxon_sep)[0] for taxon in taxa]):
+                    no_domain.append(ref_leaf.lineage)
+                    continue
+                ref_leaf.lineage = self.root_taxon + self.lin_sep + ref_leaf.lineage
+                taxa = ref_leaf.lineage.split(self.lin_sep)
             while taxa:
                 taxon_name = taxa.pop(0)  # type: str
                 rank_prefix = taxon_name.split(self.taxon_sep)[0]
@@ -759,6 +790,11 @@ class TaxonomicHierarchy:
             # Update the number of lineages provided to TaxonomicHierarchy
             self.lineages_fed += 1
             previous = None
+
+        if len(no_domain) >= 1:
+            logging.warning("Unable to load following reference package taxonomic lineages as domains are missing.\n"
+                            "{}\n\t".format("\n\t".join(no_domain)))
+
         return
 
     def emit(self, prefix_taxon: str, with_prefix=False) -> str:
@@ -890,7 +926,7 @@ class TaxonomicHierarchy:
             lineage_split = lineage_split[:-1]
         return ref_lineage
 
-    def project_lineage(self, lineage_str):
+    def project_lineage(self, lineage_str) -> bool:
         """
         Function for checking whether all taxa in a lineage exist within TaxonomicHierarchy
 
@@ -981,6 +1017,8 @@ class TaxonomicHierarchy:
         """
         reconstructed_lineage = []
         self.validate_rank_prefixes()
+        prefix = ""
+        name = ""
         for taxon in lineage.split(self.lin_sep):  # type: str
             try:
                 prefix, name = taxon.split(self.taxon_sep)
@@ -992,8 +1030,6 @@ class TaxonomicHierarchy:
                     self.so_long_and_thanks_for_all_the_fish("Rank-prefix required for clean_lineage_string().\n"
                                                              "None was provided for taxon '{}' in lineage {}\n"
                                                              "".format(taxon, lineage))
-                    raise ValueError()
-
             try:
                 if not self.no_rank_re.match(taxon) and self.rank_prefix_map[prefix] in self.accepted_ranks_depths:
                     if with_prefix is False:
