@@ -6,7 +6,7 @@ import logging
 from glob import glob
 from datetime import datetime as dt
 
-from treesapp.classy import Evaluator, Creator, PhyTrainer, Updater, Purity
+from treesapp.classy import Evaluator, Creator, PhyTrainer, Updater
 from treesapp.utilities import available_cpu_count
 
 
@@ -29,13 +29,16 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
         self.reqs = self.add_argument_group("Required parameters")
         self.seqops = self.add_argument_group("Sequence operation arguments")
         self.pplace_args = self.add_argument_group("Phylogenetic placement arguments")
-        self.rpkm_opts = self.add_argument_group("RPKM options")
+        self.fpkm_opts = self.add_argument_group("Abundance options")
         self.io = self.add_argument_group("Inputs and Outputs")
         self.aes = self.add_argument_group("Aesthetic options")
         self.optopt = self.add_argument_group("Optional options")
         self.taxa_args = self.add_argument_group("Taxonomic-lineage arguments")
+        self.svc_opts = self.add_argument_group("Classifier arguments")
         self.miscellany = self.add_argument_group("Miscellaneous options")
 
+        self.miscellany.add_argument('--overwrite', action='store_true', default=False,
+                                     help='Overwrites previously written output files and directories')
         self.miscellany.add_argument("-v", "--verbose", action="store_true", default=False,
                                      help="Prints a more verbose runtime log")
         self.miscellany.add_argument("-h", "--help",
@@ -98,12 +101,14 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                  help="Type of input sequences "
                                       "(prot = protein; dna = nucleotide [DEFAULT]; rrna = rRNA)")
 
-    def add_rpkm_params(self):
-        self.rpkm_opts.add_argument("-r", "--reads", required=False,
+    def add_abundance_params(self):
+        self.fpkm_opts.add_argument("--metric", required=False, default="fpkm", choices=["fpkm", "tpm"],
+                                    help="Selects which normalization metric to use, FPKM or TPM.")
+        self.fpkm_opts.add_argument("-r", "--reads", required=False, nargs='+',
                                     help="FASTQ file containing to be aligned to predicted genes using BWA MEM")
-        self.rpkm_opts.add_argument("-2", "--reverse", required=False,
+        self.fpkm_opts.add_argument("-2", "--reverse", required=False, nargs='+',
                                     help="FASTQ file containing to reverse mate-pair reads to be aligned using BWA MEM")
-        self.rpkm_opts.add_argument("-p", "--pairing", required=False, default='pe', choices=['pe', 'se'],
+        self.fpkm_opts.add_argument("-p", "--pairing", required=False, default='pe', choices=['pe', 'se'],
                                     help="Indicating whether the reads are paired-end (pe) or single-end (se)")
 
     def add_search_params(self):
@@ -126,12 +131,15 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
         # Doesn't _really_ fit in here but good enough. Needs to be used by create and update.
         self.optopt.add_argument("--outdet_align", default=False, action="store_true", dest="od_seq",
                                  help="Flag to activate outlier detection and removal from multiple sequence alignments"
-                                      " using OD-seq. [DEFAULT = False]")
+                                      " using OD-seq. [ DEFAULT = False ]")
 
     def add_pplace_params(self):
         self.pplace_args.add_argument("-l", "--min_like_weight_ratio", default=0.1, type=float, dest="min_lwr",
                                       help="The minimum likelihood weight ratio required for an EPA placement. "
-                                           "[DEFAULT = 0.1]")
+                                           "[ DEFAULT = 0.1 ]")
+        self.pplace_args.add_argument("--max_pendant_length", default=2.0, type=float, dest="max_pd",
+                                      help="The maximum pendant length distance threshold, "
+                                           "beyond which EPA placements are unclassified. [ DEFAULT = 2.0 ]")
         self.pplace_args.add_argument("--placement_summary", default="max_lwr", choices=["aelw", "max_lwr"],
                                       dest="p_sum",
                                       help="Controls the algorithm for consolidating multiple phylogenetic placements."
@@ -139,11 +147,9 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                            "aELW uses the taxon with greatest accumulated LWR across placements.")
 
     def add_compute_miscellany(self):
-        self.miscellany.add_argument('--overwrite', action='store_true', default=False,
-                                     help='overwrites previously processed output folders')
         self.miscellany.add_argument('-n', '--num_procs', dest="num_threads", default=2, type=int,
                                      help='The number of CPU threads or parallel processes '
-                                          'to use in various pipeline steps [DEFAULT = 2]')
+                                          'to use in various pipeline steps [ DEFAULT = 2 ]')
 
     def add_accession_params(self):
         self.taxa_args.add_argument("--accession2taxid", dest="acc_to_taxid", required=False, default=None,
@@ -195,21 +201,25 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                       " [ DEFAULT = class species ]",
                                  choices=["domain", "phylum", "class", "order", "family", "genus", "species"])
 
-    def add_classifier_model_params(self):
-        self.optopt.add_argument("--max_examples", required=False, default=1E3, type=int,
-                                 help="Limits the number of examples used for training models. [ DEFAULT = 1E3 ]")
-        self.optopt.add_argument("-k", "--svm_kernel", required=False, default="lin",
-                                 choices=["lin", "rbf", "poly"], dest="kernel",
-                                 help="Specifies the kernel type to be used in the SVM algorithm. "
-                                      "It must be either 'lin' 'poly' or 'rbf'. [ DEFAULT = lin ]")
-        self.optopt.add_argument("--grid_search", default=False, required=False, action="store_true",
-                                 help="Perform a grid search across hyperparameters. Binary classifier only.")
-        self.optopt.add_argument("--tsne", default=False, required=False, action="store_true",
-                                 help="Generate a tSNE plot. Output will be in the same directory as the model file. "
-                                      "Binary classifier only.")
-        self.optopt.add_argument("--classifier", required=False, choices=["occ", "bin"], default="occ",
-                                 help="Specify the kind of classifier to be trained: one-class classifier (OCC) or "
-                                      "a binary classifier (bin). [ DEFAULT = occ ]")
+    def add_basic_classifier_model_params(self):
+        self.svc_opts.add_argument("--max_examples", required=False, default=1000, type=int,
+                                   help="Limits the number of examples used for training models. [ DEFAULT = 1000 ]")
+        self.svc_opts.add_argument("-k", "--svm_kernel", required=False, default="lin",
+                                   choices=["lin", "rbf", "poly"], dest="kernel",
+                                   help="Specifies the kernel type to be used in the SVM algorithm. "
+                                        "It must be either 'lin' 'poly' or 'rbf'. [ DEFAULT = lin ]")
+        return
+
+    def add_advanced_classifier_params(self):
+        self.svc_opts.add_argument("--grid_search", default=False, required=False, action="store_true",
+                                   help="Perform a grid search across hyperparameters. Binary classifier only.")
+        self.svc_opts.add_argument("--tsne", default=False, required=False, action="store_true",
+                                   help="Generate a tSNE plot. Output will be in the same directory as the model file. "
+                                        "Binary classifier only.")
+        self.svc_opts.add_argument("--classifier", required=False, choices=["occ", "bin"], default="occ",
+                                   help="Specify the kind of classifier to be trained: one-class classifier (OCC) or "
+                                        "a binary classifier (bin). [ DEFAULT = occ ]")
+        return
 
 
 def add_info_arguments(parser: TreeSAPPArgumentParser):
@@ -227,8 +237,6 @@ def add_package_arguments(pkg_parser: TreeSAPPArgumentParser, attributes: list):
     pkg_parser.optopt.add_argument('-o', '--output', default="./", required=False,
                                    help='Path to an output directory. '
                                         'Default is the current working directory.')
-    pkg_parser.optopt.add_argument("--overwrite", default=False, required=False, action="store_true",
-                                   help="When editing a reference package, should the current file be overwritten?")
     return
 
 
@@ -290,7 +298,7 @@ def add_classify_arguments(assign_parser: TreeSAPPArgumentParser) -> None:
     assign_parser.add_io()
     assign_parser.add_refpkg_opt()
     assign_parser.add_refpkg_targets()
-    assign_parser.add_rpkm_params()
+    assign_parser.add_abundance_params()
     assign_parser.add_seq_params()
     assign_parser.add_search_params()
     assign_parser.add_pplace_params()
@@ -307,8 +315,9 @@ def add_classify_arguments(assign_parser: TreeSAPPArgumentParser) -> None:
     assign_parser.optopt.add_argument("--stage", default="continue", required=False,
                                       choices=["continue", "orf-call", "search", "align", "place", "classify"],
                                       help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
-    assign_parser.rpkm_opts.add_argument("--rpkm", action="store_true", default=False,
-                                         help="Flag indicating RPKM values should be calculated for the sequences detected")
+    assign_parser.fpkm_opts.add_argument("--rel_abund", action="store_true", default=False,
+                                         help="Flag indicating relative abundance values should be"
+                                              " calculated for the sequences detected")
 
     # The miscellany
     assign_parser.miscellany.add_argument('-R', '--reftree', required=False, default="", type=str,
@@ -318,17 +327,20 @@ def add_classify_arguments(assign_parser: TreeSAPPArgumentParser) -> None:
 
 
 def add_abundance_arguments(parser: TreeSAPPArgumentParser):
-    parser.add_rpkm_params()
+    parser.add_abundance_params()
     parser.add_compute_miscellany()
     parser.add_delete()
     parser.reqs.add_argument("--treesapp_output", dest="output", required=True,
                              help="Path to the directory containing TreeSAPP outputs, "
                                   "including sequences to be used for the update.")
-    # TODO: Include an option to append new values to the classification table
-    parser.optopt.add_argument("--report", choices=["update", "nothing"], required=False, default="update",
+    parser.optopt.add_argument("--report", choices=["update", "nothing", "append"], required=False, default="append",
                                help="What should be done with the abundance values? The TreeSAPP classification table "
-                                    "can be overwritten (update) or left unchanged. "
-                                    "[ DEFAULT = update ]")
+                                    "can be overwritten (update), appended or left unchanged. "
+                                    "[ DEFAULT = append ]")
+    parser.optopt.add_argument("--stage", default="continue", required=False,
+                               choices=["continue", "align_map", "sam_sum", "summarise"],
+                               help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
+    return
 
 
 def add_create_arguments(parser: TreeSAPPArgumentParser) -> None:
@@ -345,6 +357,7 @@ def add_create_arguments(parser: TreeSAPPArgumentParser) -> None:
     parser.add_phylogeny_params()
     parser.add_accession_params()
     parser.add_compute_miscellany()
+    parser.add_basic_classifier_model_params()
 
     parser.reqs.add_argument("-c", "--refpkg_name",
                              help="Unique name to be used by TreeSAPP internally. NOTE: Must be <=6 characters.\n"
@@ -474,7 +487,8 @@ def add_trainer_arguments(parser: TreeSAPPArgumentParser) -> None:
     parser.add_accession_params()
     parser.add_taxa_ranks_param()
     parser.add_compute_miscellany()
-    parser.add_classifier_model_params()
+    parser.add_basic_classifier_model_params()
+    parser.add_advanced_classifier_params()
     parser.add_annot_map()
 
     parser.seqops.add_argument("-d", "--profile", required=False, default=False, action="store_true",
@@ -523,25 +537,6 @@ def check_parser_arguments(args, sys_args):
     return
 
 
-def check_purity_arguments(purity_instance: Purity, args):
-    purity_instance.ref_pkg.f__json = args.pkg_path
-    purity_instance.ref_pkg.slurp()
-    purity_instance.refpkg_dir = os.path.dirname(purity_instance.ref_pkg.f__json)
-
-    ##
-    # Define locations of files TreeSAPP outputs
-    ##
-    purity_instance.assign_dir = purity_instance.var_output_dir + "assign" + os.sep
-    purity_instance.summarize_dir = purity_instance.var_output_dir + "summarize" + os.sep
-    purity_instance.classifications = purity_instance.assign_dir + "final_outputs" + os.sep + "marker_contig_map.tsv"
-    purity_instance.metadata_file = args.extra_info
-
-    if not os.path.isdir(purity_instance.var_output_dir):
-        os.makedirs(purity_instance.var_output_dir)
-
-    return
-
-
 def check_evaluate_arguments(evaluator_instance: Evaluator, args) -> None:
     for rank in args.taxon_rank:
         evaluator_instance.ranks.append(rank)
@@ -554,16 +549,6 @@ def check_evaluate_arguments(evaluator_instance: Evaluator, args) -> None:
     else:
         evaluator_instance.min_seq_length = str(30)
 
-    if args.acc_to_lin:
-        evaluator_instance.acc_to_lin = args.acc_to_lin
-        if os.path.isfile(evaluator_instance.acc_to_lin):
-            evaluator_instance.change_stage_status("lineages", False)
-        else:
-            logging.error("Unable to find accession-lineage mapping file '{}'\n".format(evaluator_instance.acc_to_lin))
-            sys.exit(3)
-    else:
-        evaluator_instance.acc_to_lin = evaluator_instance.var_output_dir + os.sep + "accession_id_lineage_map.tsv"
-
     ##
     # Define locations of files TreeSAPP outputs
     ##
@@ -571,7 +556,6 @@ def check_evaluate_arguments(evaluator_instance: Evaluator, args) -> None:
     evaluator_instance.performance_table = evaluator_instance.final_output_dir + "clade_exclusion_performance.tsv"
     evaluator_instance.recall_table = evaluator_instance.final_output_dir + "taxonomic_recall.tsv"
     evaluator_instance.containment_table = evaluator_instance.final_output_dir + "accuracy.tsv"
-    evaluator_instance.var_output_dir = args.output + "intermediates" + os.sep
 
     if not os.path.isdir(evaluator_instance.var_output_dir):
         os.makedirs(evaluator_instance.var_output_dir)
