@@ -7,9 +7,10 @@ import logging
 from shutil import copy, rmtree
 
 from tqdm import tqdm
+import subprocess
 
 from treesapp.external_command_interface import launch_write_command, CommandLineFarmer
-from treesapp.fasta import read_fasta_to_dict
+from treesapp import utilities
 
 
 def estimate_ml_model(modeltest_exe: str, msa: str, output_prefix: str, molecule: str, threads=1) -> str:
@@ -68,20 +69,26 @@ def model_parameters(raxml_exe: str, ref_msa: str, tree_file: str, output_prefix
     """
     output_prefix += "_evaluate"
     model_params_file = output_prefix + ".raxml.bestModel"
+    ml_tree = output_prefix + ".raxml.bestTree"
     model_eval_cmd = [raxml_exe, "--evaluate"]
-    model_eval_cmd += ["--msa", ref_msa]
-    model_eval_cmd += ["--tree", tree_file]
-    model_eval_cmd += ["--prefix", output_prefix]
+    model_eval_cmd += ["--msa", "'" + ref_msa + "'"]
+    model_eval_cmd += ["--tree", "'" + tree_file + "'"]
+    model_eval_cmd += ["--prefix", "'" + output_prefix + "'"]
     model_eval_cmd += ["--model", model]
     model_eval_cmd += ["--threads", "auto{{{}}}".format(threads)]
     model_eval_cmd += ["--seed", str(12345)]
     model_eval_cmd += ["--workers", "auto{{{}}}".format(threads)]
     model_eval_cmd.append("--force")
+    if glob.glob(utilities.globify_path(output_prefix) + "*"):
+        model_eval_cmd.append("--redo")
 
     logging.debug("Evaluating phylogenetic tree with RAxML-NG... ")
     stdout, returncode = launch_write_command(model_eval_cmd)
     logging.debug("done.\n")
 
+    # Overwrite the original tree with the RAxML-NG evaluated tree
+    os.rename(ml_tree, tree_file)
+
     if returncode != 0:
         logging.error("{} did not complete successfully! Look in {}_info.txt for an error message.\n"
                       "RAxML-NG command used:\n{}\n".format(raxml_exe, output_prefix, ' '.join(model_eval_cmd)))
@@ -92,6 +99,11 @@ def model_parameters(raxml_exe: str, ref_msa: str, tree_file: str, output_prefix
 
 def bootstrap_tree_raxml(raxml_exe: str, multiple_alignment: str, model: str, tree_prefix: str,
                          mre=True, bootstraps=1000, num_threads=2) -> str:
+    if mre and bootstraps < 100:
+        logging.warning("With fewer than 100 bootstraps specified ({}) "
+                        "MRE-based bootstrap convergence criterion in not going to be used.".format(bootstraps))
+        mre = False
+
     bootstrap_cmd = [raxml_exe, "--bootstrap"]
     bootstrap_cmd += ["--msa", multiple_alignment]
     bootstrap_cmd += ["--model", model]
@@ -125,7 +137,7 @@ def support_tree_raxml(raxml_exe: str, ref_tree: str, ref_msa: str, model: str,
     support_cmd += ["--prefix", tree_prefix]
     support_cmd += ["--threads", str(num_threads)]
 
-    logging.info("Calculating bootstrap support for node in reference tree with RAxML-NG... ")
+    logging.info("Calculating bootstrap support for nodes in reference tree with RAxML-NG... ")
     launch_write_command(support_cmd)
     logging.info("done.\n")
 
@@ -155,8 +167,8 @@ def construct_tree(tree_builder: str, executables: dict, evo_model: str, multipl
 
     # Decide on the command to build the tree, make some directories and files when necessary
     logging.info("Building phylogenetic tree with " + tree_builder + "... ")
+    best_tree = "{}{}.{}.nwk".format(tree_output_dir, tree_prefix, tree_builder)
     if tree_builder == "FastTree":
-        best_tree = tree_output_dir + tree_prefix + ".FastTree.nwk"
         tree_build_cmd = [executables["FastTree"]]
         if re.search(r"GTR", evo_model):
             tree_build_cmd += ["-nt", "-gtr"]
@@ -169,17 +181,17 @@ def construct_tree(tree_builder: str, executables: dict, evo_model: str, multipl
         with open(tree_output_dir + tree_prefix + ".FastTree.log", 'w') as fast_info:
             fast_info.write(stdout + "\n")
     elif tree_builder == "RAxML-NG":
-        best_tree = tree_output_dir + tree_prefix + ".raxml.bestTree"
         tree_build_cmd = [executables["raxml-ng"], "--search"]
         tree_build_cmd += ["--prefix", tree_output_dir + tree_prefix]
         tree_build_cmd += ["--msa", multiple_alignment_file]
         tree_build_cmd += ["--model", evo_model]
-        # tree_build_cmd += ["--msa-format", "PHYLIP"]  # File isn't read properly with this parameter, use auto-detect
         tree_build_cmd += ["--seed", str(12345)]
         tree_build_cmd += ["--threads", str(num_threads)]
         # tree_build_cmd += ["--tree", "rand{1},pars{1}"]  # For debugging, alternatively could use '--search1'
 
         stdout, returncode = launch_write_command(tree_build_cmd)
+        # Rename the file using the standardised naming scheme
+        os.rename(tree_output_dir + tree_prefix + ".raxml.bestTree", best_tree)
     else:
         logging.error("Unrecognized software '{}'.\n".format(tree_builder))
         sys.exit(5)
@@ -600,7 +612,7 @@ def build_hmm_profile(hmmbuild_exe: str, msa_in: str, output_hmm: str, name=None
     hmm_build_command = [hmmbuild_exe]
     if name:
         hmm_build_command += ["-n", str(name)]
-    hmm_build_command += [output_hmm, msa_in]
+    hmm_build_command += ["'" + output_hmm + "'", "'" + msa_in + "'"]
     stdout, hmmbuild_pro_returncode = launch_write_command(hmm_build_command)
     logging.debug("done.\n")
 
@@ -634,8 +646,8 @@ def run_hmmsearch(hmmsearch_exe: str, hmm_profile: str, query_fasta: str, output
     hmmsearch_command_base += ["-E", str(e_value)]
     hmmsearch_command_base.append("--noali")
     # Customize the command for this input and HMM
-    final_hmmsearch_command = hmmsearch_command_base + ["--domtblout", domtbl]
-    final_hmmsearch_command += [hmm_profile, query_fasta]
+    final_hmmsearch_command = hmmsearch_command_base + ["--domtblout", "'" + domtbl + "'"]
+    final_hmmsearch_command += ["'" + hmm_profile + "'", "'" + query_fasta + "'"]
     stdout, ret_code = launch_write_command(final_hmmsearch_command)
 
     # Check to ensure the job finished properly
@@ -688,7 +700,67 @@ def hmmsearch_orfs(hmmsearch_exe: str, refpkg_dict: dict, fasta_file: str, outpu
     return hmm_domtbl_files
 
 
-def run_mafft(mafft_exe: str, fasta_in: str, fasta_out: str, num_threads) -> None:
+def align_reads_to_nucs(bwa_exe: str, reference_fasta: str, aln_output_dir: str,
+                        reads: str, pairing: str, reverse=None, num_threads=2) -> str:
+    """
+    Align the predicted ORFs to the reads using BWA MEM
+
+    :param bwa_exe: Path to the BWA executable
+    :param reference_fasta: A FASTA file containing the sequences to be aligned to
+    :param aln_output_dir: Path to the directory to write the index and SAM files
+    :param reads: FASTQ file containing reads to be aligned to the reference FASTA file
+    :param pairing: Either 'se' or 'pe' indicating the reads are single-end or paired-end, respectively
+    :param reverse: Path to reverse-orientation mate pair reads [OPTIONAL]
+    :param num_threads: Number of threads for BWA MEM to use
+    :return: Path to the SAM file
+    """
+    if not os.path.exists(aln_output_dir):
+        try:
+            os.makedirs(aln_output_dir)
+        except OSError:
+            if os.path.exists(aln_output_dir):
+                logging.warning("Overwriting files in " + aln_output_dir + ".\n")
+            else:
+                raise OSError("Unable to make " + aln_output_dir + "!\n")
+
+    logging.info("Aligning reads to ORFs with BWA MEM... ")
+
+    sam_file = aln_output_dir + '.'.join(os.path.basename(reference_fasta).split('.')[0:-1]) + ".sam"
+    if os.path.isfile(sam_file):
+        logging.info("Alignment map file {} found.\n".format(sam_file))
+        return sam_file
+    index_command = [bwa_exe, "index"]
+    index_command += [reference_fasta]
+    index_command += ["1>", "/dev/null", "2>", aln_output_dir + "treesapp_bwa_index.stderr"]
+
+    launch_write_command(index_command)
+
+    bwa_command = [bwa_exe, "mem"]
+    bwa_command += ["-t", str(num_threads)]
+    if pairing == "pe" and not reverse:
+        bwa_command.append("-p")
+        logging.debug("FASTQ file containing reverse mates was not provided - assuming the reads are interleaved!\n")
+    elif pairing == "se":
+        bwa_command += ["-S", "-P"]
+
+    bwa_command.append(reference_fasta)
+    bwa_command.append(reads)
+    if pairing == "pe" and reverse:
+        bwa_command.append(reverse)
+    bwa_command += ["1>", sam_file, "2>", aln_output_dir + "treesapp_bwa_mem.stderr"]
+
+    p_bwa = subprocess.Popen(' '.join(bwa_command), shell=True, preexec_fn=os.setsid)
+    p_bwa.wait()
+    if p_bwa.returncode != 0:
+        logging.error("bwa mem did not complete successfully for:\n" +
+                      str(' '.join(bwa_command)) + "\n")
+
+    logging.info("done.\n")
+
+    return sam_file
+
+
+def run_mafft(mafft_exe: str, fasta_in: str, fasta_out: str, num_threads: int) -> None:
     """
     Wrapper function for the MAFFT multiple sequence alignment tool.
     Runs MAFFT using `--auto` and checks if the output is empty.
@@ -712,12 +784,12 @@ def run_mafft(mafft_exe: str, fasta_in: str, fasta_out: str, num_threads) -> Non
         logging.error("Multiple sequence alignment using " + mafft_exe +
                       " did not complete successfully! Command used:\n" + ' '.join(mafft_align_command) + "\n")
         sys.exit(7)
-    else:
-        mfa = read_fasta_to_dict(fasta_out)
-        if len(mfa.keys()) < 1:
-            logging.error("MAFFT did not generate a proper FASTA file. " +
-                          "Check the output by running:\n" + ' '.join(mafft_align_command) + "\n")
-            sys.exit(7)
+
+    # mfa = read_fasta_to_dict(fasta_out)
+    # if len(mfa.keys()) < 1:
+    #     logging.error("MAFFT did not generate a proper FASTA file. " +
+    #                   "Check the output by running:\n" + ' '.join(mafft_align_command) + "\n")
+    #     sys.exit(7)
 
     return
 
@@ -823,3 +895,22 @@ def filter_multiple_alignments(executables, concatenated_mfa_files, refpkg_dict,
     logging.debug("\t" + tool + " time required: " +
                   ':'.join([str(hours), str(minutes), str(round(seconds, 2))]) + "\n")
     return trimmed_output_files
+
+
+def run_graftm_graft(graftm_exe: str, input_path: str, output_dir: str, gpkg_path: str,
+                     classifier="graftm", seq_type="aminoacid", num_threads=4) -> None:
+    """Wrapper for GraftM"""
+    if not os.path.isdir(output_dir):
+        os.makedirs(output_dir)
+    classify_call = [graftm_exe, "graft",
+                     "--forward", input_path,
+                     "--graftm_package", gpkg_path,
+                     "--input_sequence_type", seq_type,
+                     "--threads", str(num_threads),
+                     "--output_directory", output_dir,
+                     "--force"]
+    if classifier == "diamond":
+        classify_call += ["--assignment_method", "diamond"]
+        classify_call += ["--search_method", "diamond"]
+    launch_write_command(classify_call, False)
+    return
